@@ -24,18 +24,24 @@ m4_dnl ()
       (env, vm->class_loading.boot_loader.class_loader_info,
        "$1", vm->class_loading.boot_loader.classes.$2) != JNI_OK)
     {
+      _svmf_printf (env, stderr,
+		    "*** Fatal error: Unable to create essential class $1.\n");
       return JNI_ERR;
     }
 
   if (_svmf_link_class
       (env, vm->class_loading.boot_loader.classes.$2) != JNI_OK)
     {
+      _svmf_printf (env, stderr,
+		    "*** Fatal error: Unable to link essential class $1.\n");
       return JNI_ERR;
     }
 
   if (_svmf_special_initialization
       (env, vm->class_loading.boot_loader.classes.$2) != JNI_OK)
     {
+      _svmf_printf (env, stderr,
+		    "*** Fatal error: Unable to initialize essential class $1.\n");
       return JNI_ERR;
     }
   m4svm_define_end v = ":])";
@@ -86,6 +92,8 @@ m4_dnl ()
 
   if (vm->class_loading.boot_loader.methods.$3_$4 == NULL)
     {
+      _svmf_printf (env, stderr,
+		    "*** Fatal error: Unable to find essential method $1 in $3 class.\n");
       _svmf_error_NoSuchMethodError (env);
       return JNI_ERR;
     }
@@ -110,6 +118,8 @@ m4_dnl ()
 
   if (vm->class_loading.boot_loader.fields.$3_$4 == NULL)
     {
+      _svmf_printf (env, stderr,
+		    "*** Fatal error: Unable to find essential field $1 in $3 class.\n");
       _svmf_error_NoSuchFieldError (env);
       return JNI_ERR;
     }
@@ -201,11 +211,14 @@ _svmf_bootstrap
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_bootstrap (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
 
+  if (_svmf_bootcl_internal_split_bootcl (env) != JNI_OK)
+    return JNI_ERR;
+
   /* We must create the following classes and array first. This will
      allow us to create java/lang/Class instances.
 
@@ -237,7 +250,8 @@ _svmf_bootstrap (_svmt_JNIEnv *env)
 
   vm->class_loading.boot_loader.methods.jlclass_init =
     _svmf_resolve_method (vm->class_loading.boot_loader.classes.jlclass,
-			  "<init>", "(Ljava/lang/Object;)V");
+			  "<init>",
+			  "(Ljava/lang/Object;Ljava/security/ProtectionDomain;)V");
 
   if (vm->class_loading.boot_loader.methods.jlclass_init == NULL)
     {
@@ -345,11 +359,14 @@ _svmf_bootstrap (_svmt_JNIEnv *env)
      m4svm_off()
 
      m4svm_create_link_and_initialize_class(java/lang/StackTraceElement, jlstacktraceelement)
-     m4svm_create_link_and_initialize_class(java/lang/reflect/Constructor, jlrconstructor)
-     m4svm_create_link_and_initialize_class(java/lang/reflect/Field, jlrfield)
-     m4svm_create_link_and_initialize_class(java/lang/reflect/Method, jlrmethod)
      m4svm_create_link_and_initialize_class(java/lang/StringCreator, stringcreator)
      m4svm_create_link_and_initialize_class(java/lang/VirtualMachine, virtualmachine)
+     m4svm_create_link_and_initialize_class(java/lang/reflect/Constructor, jlrconstructor)
+     m4svm_create_link_and_initialize_class(java/lang/reflect/Field, jlrfield)
+     m4svm_create_link_and_initialize_class(java/lang/reflect/InvocationTargetException, jlrinvocationtargetexception)
+     m4svm_create_link_and_initialize_class(java/lang/reflect/Method, jlrmethod)
+
+     m4svm_create_and_link_array([Ljava/lang/reflect/Field;, jlrfield_array)
 
      m4svm_create_and_link_array([Z, boolean_array)
      m4svm_create_and_link_array([S, short_array)
@@ -399,18 +416,23 @@ _svmf_bootstrap (_svmt_JNIEnv *env)
 
      m4svm_find_method(<init>, (C)V, jlclass, init_primitive)
      m4svm_find_method(<init>, (Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;Z)V, jlstacktraceelement, init)
+     m4svm_find_method(<init>, (Ljava/lang/Throwable;)V, jlrinvocationtargetexception, init)
      m4svm_find_method(<init>, ([B)V, jlrconstructor, init)
      m4svm_find_method(<init>, ([B)V, jlrfield, init)
      m4svm_find_method(<init>, ([B)V, jlrmethod, init)
+     m4svm_find_method(createArray, (Ljava/lang/ClassLoader;Ljava/lang/String;)Ljava/lang/Class;, virtualmachine, createarray)
+     m4svm_find_method(createClass, (Ljava/lang/ClassLoader;Ljava/lang/String;)Ljava/lang/Class;, virtualmachine, createclass)
      m4svm_find_method(createInternedString, ([B)Ljava/lang/String;, stringcreator, createinternedstring)
+     m4svm_find_method(createRootThread, ([B)Ljava/lang/Thread;, virtualmachine, createrootthread)
      m4svm_find_method(createString, ([B)Ljava/lang/String;, stringcreator, createstring)
+     m4svm_find_method(createStringFromChars, ([C)Ljava/lang/String;, stringcreator, createstringfromchars)
+     m4svm_find_method(exceptionDescribe, (Ljava/lang/Throwable;)V, virtualmachine, exceptiondescribe)
+     m4svm_find_method(getChars, (Ljava/lang/String;)[C, stringcreator, getchars)
+     m4svm_find_method(getLength, (Ljava/lang/String;)I, stringcreator, getlength)
+     m4svm_find_method(getSystemClassLoader, ()[B, virtualmachine, getsystemcl)
      m4svm_find_method(getUTFChars, (Ljava/lang/String;)[B, stringcreator, getutfchars)
      m4svm_find_method(getUTFLength, (Ljava/lang/String;)I, stringcreator, getutflength)
      m4svm_find_method(initialize, (I)V, jlclass, initialize)
-     m4svm_find_method(exceptionDescribe, (Ljava/lang/Throwable;)V, virtualmachine, exceptiondescribe)
-     m4svm_find_method(createClass, (Ljava/lang/ClassLoader;Ljava/lang/String;)Ljava/lang/Class;, virtualmachine, createclass)
-     m4svm_find_method(createArray, (Ljava/lang/ClassLoader;Ljava/lang/String;)Ljava/lang/Class;, virtualmachine, createarray)
-     m4svm_find_method(createRootThread, ([B)Ljava/lang/Thread;, virtualmachine, createrootthread)
      m4svm_find_method(runThread, ()V, virtualmachine, runthread)
 
      m4svm_find_method(<init>, (Ljava/lang/String;)V, ArithmeticException, init)
