@@ -27,7 +27,7 @@ _svmh_parse_$1
 #define _svmm_parse_$1(env, current, size, result) \
 _svmh_parse_$1 (env, &current, &size, &result)
 
-static jint
+svm_static jint
 _svmh_parse_$1 (_svmt_JNIEnv *env, _svmt_u8 **pcurrent, size_t *psize,
 		$3 * presult)
 {
@@ -110,7 +110,7 @@ _svmh_parse_cp_index_$1
 #define _svmm_parse_cp_index_$1(env, current, size, class, result) \
 _svmh_parse_cp_index_$1 (env, &current, &size, class, &result)
 
-static jint
+svm_static jint
 _svmh_parse_cp_index_$1 (_svmt_JNIEnv *env, _svmt_u8 **pcurrent,
 			 size_t *psize, _svmt_class_info *class,
 			 _svmt_$1_info *** presult)
@@ -265,26 +265,40 @@ _svmh_parse_d64
 #define _svmm_parse_d64(env, current, size, result) \
 _svmh_parse_d64 (env, &current, &size, &result)
 
-static jint
+svm_static jint
 _svmh_parse_d64 (_svmt_JNIEnv *env, _svmt_u8 **pcurrent, size_t *psize,
 		 jdouble *presult)
 {
   size_t i;
-  _svmt_s64 result = 0;
+  _svmt_u64 result = 0;
 
-  if (*psize < sizeof (_svmt_s64))
+  if (*psize < sizeof (_svmt_u64))
     {
       _svmf_error_ClassFormatError (env);
       return JNI_ERR;
     }
 
-  for (i = 0; i < sizeof (_svmt_s64); i++)
+  for (i = 0; i < sizeof (_svmt_u64); i++)
     {
       result = (result << 8) | (*pcurrent)[i];
     }
 
-  *psize -= sizeof (_svmt_s64);
-  *pcurrent += sizeof (_svmt_s64);
+#if defined(__arm__)
+
+  /* On ARM 'double' values have their two 32bit parts reversed. */
+
+  {
+    _svmt_u64 result1, result2;
+
+    result1 = result & 0xffffffff;
+    result2 = result >> 32;
+    result = (result1 << 32) | result2;
+  }
+
+#endif
+
+  *psize -= sizeof (_svmt_u64);
+  *pcurrent += sizeof (_svmt_u64);
 
   *presult = *((jdouble *) &result);
   return JNI_OK;
@@ -299,7 +313,7 @@ _svmh_parse_f32
 #define _svmm_parse_f32(env, current, size, result) \
 _svmh_parse_f32 (env, &current, &size, &result)
 
-static jint
+svm_static jint
 _svmh_parse_f32 (_svmt_JNIEnv *env, _svmt_u8 **pcurrent, size_t *psize,
 		 jfloat *presult)
 {
@@ -330,11 +344,12 @@ _svmf_parse_CONSTANT_Class_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Class_info (_svmt_JNIEnv *env,
 				 _svmt_class_loader_info
-				 *class_loader_info, _svmt_u8 **pcurrent,
-				 size_t *psize, _svmt_class_info *class,
+				 *class_loader_info SVM_UNUSED,
+				 _svmt_u8 **pcurrent, size_t *psize,
+				 _svmt_class_info *class,
 				 _svmt_CONSTANT_Class_info *info)
 {
   if (_svmm_parse_cp_index_CONSTANT_Utf8
@@ -352,11 +367,12 @@ _svmf_parse_CONSTANT_Fieldref_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Fieldref_info (_svmt_JNIEnv *env,
 				    _svmt_class_loader_info
-				    *class_loader_info, _svmt_u8 **pcurrent,
-				    size_t *psize, _svmt_class_info *class,
+				    *class_loader_info SVM_UNUSED,
+				    _svmt_u8 **pcurrent, size_t *psize,
+				    _svmt_class_info *class,
 				    _svmt_CONSTANT_Fieldref_info *info)
 {
   if (_svmm_parse_cp_index_CONSTANT_Class
@@ -380,11 +396,12 @@ _svmf_parse_CONSTANT_Methodref_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Methodref_info (_svmt_JNIEnv *env,
 				     _svmt_class_loader_info
-				     *class_loader_info, _svmt_u8 **pcurrent,
-				     size_t *psize, _svmt_class_info *class,
+				     *class_loader_info SVM_UNUSED,
+				     _svmt_u8 **pcurrent, size_t *psize,
+				     _svmt_class_info *class,
 				     _svmt_CONSTANT_Methodref_info *info)
 {
   if (_svmm_parse_cp_index_CONSTANT_Class
@@ -408,10 +425,10 @@ _svmf_parse_CONSTANT_InterfaceMethodref_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_InterfaceMethodref_info (_svmt_JNIEnv *env,
 					      _svmt_class_loader_info
-					      *class_loader_info,
+					      *class_loader_info SVM_UNUSED,
 					      _svmt_u8 **pcurrent,
 					      size_t *psize,
 					      _svmt_class_info *class,
@@ -439,11 +456,12 @@ _svmf_parse_CONSTANT_String_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_String_info (_svmt_JNIEnv *env,
 				  _svmt_class_loader_info
-				  *class_loader_info, _svmt_u8 **pcurrent,
-				  size_t *psize, _svmt_class_info *class,
+				  *class_loader_info SVM_UNUSED,
+				  _svmt_u8 **pcurrent, size_t *psize,
+				  _svmt_class_info *class,
 				  _svmt_CONSTANT_String_info *info)
 {
   if (_svmm_parse_cp_index_CONSTANT_Utf8
@@ -452,6 +470,8 @@ _svmf_parse_CONSTANT_String_info (_svmt_JNIEnv *env,
       return JNI_ERR;
     }
 
+  info->class_info = class;
+
   return JNI_OK;
 }
 
@@ -461,12 +481,12 @@ _svmf_parse_CONSTANT_Integer_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Integer_info (_svmt_JNIEnv *env,
 				   _svmt_class_loader_info
-				   *class_loader_info, _svmt_u8 **pcurrent,
-				   size_t *psize,
-				   _svmt_class_info *class,
+				   *class_loader_info SVM_UNUSED,
+				   _svmt_u8 **pcurrent, size_t *psize,
+				   _svmt_class_info *class SVM_UNUSED,
 				   _svmt_CONSTANT_Integer_info *info)
 {
   if (_svmm_parse_s32 (env, *pcurrent, *psize, info->value) != JNI_OK)
@@ -483,12 +503,12 @@ _svmf_parse_CONSTANT_Float_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Float_info (_svmt_JNIEnv *env,
 				 _svmt_class_loader_info
-				 *class_loader_info, _svmt_u8 **pcurrent,
-				 size_t *psize,
-				 _svmt_class_info *class,
+				 *class_loader_info SVM_UNUSED,
+				 _svmt_u8 **pcurrent, size_t *psize,
+				 _svmt_class_info *class SVM_UNUSED,
 				 _svmt_CONSTANT_Float_info *info)
 {
   if (_svmm_parse_f32 (env, *pcurrent, *psize, info->value) != JNI_OK)
@@ -505,12 +525,12 @@ _svmf_parse_CONSTANT_Long_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Long_info (_svmt_JNIEnv *env,
 				_svmt_class_loader_info
-				*class_loader_info, _svmt_u8 **pcurrent,
-				size_t *psize,
-				_svmt_class_info *class,
+				*class_loader_info SVM_UNUSED,
+				_svmt_u8 **pcurrent, size_t *psize,
+				_svmt_class_info *class SVM_UNUSED,
 				_svmt_CONSTANT_Long_info *info)
 {
   if (_svmm_parse_s64 (env, *pcurrent, *psize, info->value) != JNI_OK)
@@ -527,12 +547,12 @@ _svmf_parse_CONSTANT_Double_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Double_info (_svmt_JNIEnv *env,
 				  _svmt_class_loader_info
-				  *class_loader_info, _svmt_u8 **pcurrent,
-				  size_t *psize,
-				  _svmt_class_info *class,
+				  *class_loader_info SVM_UNUSED,
+				  _svmt_u8 **pcurrent, size_t *psize,
+				  _svmt_class_info *class SVM_UNUSED,
 				  _svmt_CONSTANT_Double_info *info)
 {
   if (_svmm_parse_d64 (env, *pcurrent, *psize, info->value) != JNI_OK)
@@ -549,10 +569,10 @@ _svmf_parse_CONSTANT_NameAndType_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_NameAndType_info (_svmt_JNIEnv *env,
 				       _svmt_class_loader_info
-				       *class_loader_info,
+				       *class_loader_info SVM_UNUSED,
 				       _svmt_u8 **pcurrent, size_t *psize,
 				       _svmt_class_info *class,
 				       _svmt_CONSTANT_NameAndType_info *info)
@@ -578,11 +598,11 @@ _svmf_parse_CONSTANT_Utf8_info
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_CONSTANT_Utf8_info (_svmt_JNIEnv *env,
 				_svmt_class_loader_info *class_loader_info,
 				_svmt_u8 **pcurrent, size_t *psize,
-				_svmt_class_info *class,
+				_svmt_class_info *class SVM_UNUSED,
 				_svmt_CONSTANT_Utf8_info *info)
 {
   if (_svmm_parse_u16 (env, *pcurrent, *psize, info->length) != JNI_OK)
@@ -636,11 +656,11 @@ _svmh_parse_constant_pool_entry
 #define _svmm_parse_constant_pool_entry(env,cli,current,size,class,entry) \
 _svmh_parse_constant_pool_entry (env, cli, &current, &size, class, &entry)
 
-static jint
+svm_static jint
 _svmh_parse_constant_pool_entry (_svmt_JNIEnv *env,
-				 _svmt_class_loader_info *class_loader_info,
-				 _svmt_u8 **pcurrent, size_t *psize,
-				 _svmt_class_info *class,
+				 _svmt_class_loader_info *class_loader_info
+				 SVM_UNUSED, _svmt_u8 **pcurrent,
+				 size_t *psize, _svmt_class_info *class,
 				 _svmt_cp_info **pentry)
 {
   jint tag;
@@ -689,12 +709,13 @@ _svmf_parse_ConstantValue_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_ConstantValue_attribute (_svmt_JNIEnv *env,
 				     _svmt_class_loader_info
-				     *class_loader_info, _svmt_u8 **pcurrent,
-				     size_t *psize,
-				     jlong length,
+				     *class_loader_info SVM_UNUSED,
+				     _svmt_u8 **pcurrent,
+				     size_t *psize SVM_UNUSED,
+				     jlong length SVM_UNUSED,
 				     _svmt_class_info *class,
 				     _svmt_ConstantValue_attribute *attribute)
 {
@@ -713,15 +734,16 @@ _svmf_parse_Synthetic_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
-_svmf_parse_Synthetic_attribute (_svmt_JNIEnv *env,
+svm_static jint
+_svmf_parse_Synthetic_attribute (_svmt_JNIEnv *env SVM_UNUSED,
 				 _svmt_class_loader_info
-				 *class_loader_info,
-				 _svmt_u8 **pcurrent,
-				 size_t *psize,
-				 jlong length,
-				 _svmt_class_info *class,
-				 _svmt_Synthetic_attribute *attribute)
+				 *class_loader_info SVM_UNUSED,
+				 _svmt_u8 **pcurrent SVM_UNUSED,
+				 size_t *psize SVM_UNUSED,
+				 jlong length SVM_UNUSED,
+				 _svmt_class_info *class SVM_UNUSED,
+				 _svmt_Synthetic_attribute *attribute
+				 SVM_UNUSED)
 {
   return JNI_OK;
 }
@@ -732,15 +754,16 @@ _svmf_parse_Deprecated_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
-_svmf_parse_Deprecated_attribute (_svmt_JNIEnv *env,
+svm_static jint
+_svmf_parse_Deprecated_attribute (_svmt_JNIEnv *env SVM_UNUSED,
 				  _svmt_class_loader_info
-				  *class_loader_info,
-				  _svmt_u8 **pcurrent,
-				  size_t *psize,
-				  jlong length,
-				  _svmt_class_info *class,
-				  _svmt_Deprecated_attribute *attribute)
+				  *class_loader_info SVM_UNUSED,
+				  _svmt_u8 **pcurrent SVM_UNUSED,
+				  size_t *psize SVM_UNUSED,
+				  jlong length SVM_UNUSED,
+				  _svmt_class_info *class SVM_UNUSED,
+				  _svmt_Deprecated_attribute *attribute
+				  SVM_UNUSED)
 {
   return JNI_OK;
 }
@@ -751,13 +774,13 @@ _svmf_parse_unknown_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_unknown_attribute (_svmt_JNIEnv *env,
 			       _svmt_class_loader_info *class_loader_info,
 			       _svmt_u8 **pcurrent,
 			       size_t *psize,
 			       jlong length,
-			       _svmt_class_info *class,
+			       _svmt_class_info *class SVM_UNUSED,
 			       _svmt_unknown_attribute *attribute)
 {
   if (*psize < (size_t) length)
@@ -788,14 +811,14 @@ _svmf_parse_LineNumberTable_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_LineNumberTable_attribute (_svmt_JNIEnv *env,
 				       _svmt_class_loader_info
 				       *class_loader_info,
 				       _svmt_u8 **pcurrent, size_t *psize,
-				       jlong length,
+				       jlong length SVM_UNUSED,
 				       _svmt_class_info
-				       *class,
+				       *class SVM_UNUSED,
 				       _svmt_LineNumberTable_attribute
 				       *attribute)
 {
@@ -846,12 +869,12 @@ _svmf_parse_LocalVariableTable_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_LocalVariableTable_attribute (_svmt_JNIEnv *env,
 					  _svmt_class_loader_info
 					  *class_loader_info,
 					  _svmt_u8 **pcurrent, size_t *psize,
-					  jlong length,
+					  jlong length SVM_UNUSED,
 					  _svmt_class_info *class,
 					  _svmt_LocalVariableTable_attribute
 					  *attribute)
@@ -926,12 +949,12 @@ _svmf_parse_Exceptions_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_Exceptions_attribute (_svmt_JNIEnv *env,
 				  _svmt_class_loader_info
 				  *class_loader_info,
 				  _svmt_u8 **pcurrent, size_t *psize,
-				  jlong length,
+				  jlong length SVM_UNUSED,
 				  _svmt_class_info *class,
 				  _svmt_Exceptions_attribute *attribute)
 {
@@ -974,12 +997,12 @@ _svmf_parse_InnerClasses_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_InnerClasses_attribute (_svmt_JNIEnv *env,
 				    _svmt_class_loader_info
 				    *class_loader_info,
 				    _svmt_u8 **pcurrent, size_t *psize,
-				    jlong length,
+				    jlong length SVM_UNUSED,
 				    _svmt_class_info *class,
 				    _svmt_InnerClasses_attribute *attribute)
 {
@@ -1047,12 +1070,12 @@ _svmf_parse_SourceFile_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_SourceFile_attribute (_svmt_JNIEnv *env,
 				  _svmt_class_loader_info
-				  *class_loader_info, _svmt_u8 **pcurrent,
-				  size_t *psize,
-				  jlong length,
+				  *class_loader_info SVM_UNUSED,
+				  _svmt_u8 **pcurrent, size_t *psize,
+				  jlong length SVM_UNUSED,
 				  _svmt_class_info *class,
 				  _svmt_SourceFile_attribute *attribute)
 {
@@ -1074,7 +1097,7 @@ _svmh_parse_code_attribute
 #define _svmm_parse_code_attribute(env,cli,current,size,class,attribute) \
 _svmh_parse_code_attribute (env, cli, &current, &size, class, &attribute)
 
-static jint
+svm_static jint
 _svmh_parse_code_attribute (_svmt_JNIEnv *env,
 			    _svmt_class_loader_info *class_loader_info,
 			    _svmt_u8 **pcurrent, size_t *psize,
@@ -1134,12 +1157,12 @@ _svmf_parse_Code_attribute
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_Code_attribute (_svmt_JNIEnv *env,
 			    _svmt_class_loader_info *class_loader_info,
 			    _svmt_u8 **pcurrent,
 			    size_t *psize,
-			    jlong length,
+			    jlong length SVM_UNUSED,
 			    _svmt_class_info *class,
 			    _svmt_Code_attribute *attribute)
 {
@@ -1278,7 +1301,7 @@ _svmh_parse_field_attribute
 #define _svmm_parse_field_attribute(env,cli,current,size,class,attribute) \
 _svmh_parse_field_attribute (env, cli, &current, &size, class, &attribute)
 
-static jint
+svm_static jint
 _svmh_parse_field_attribute (_svmt_JNIEnv *env,
 			     _svmt_class_loader_info *class_loader_info,
 			     _svmt_u8 **pcurrent, size_t *psize,
@@ -1342,7 +1365,7 @@ _svmh_parse_method_attribute
 #define _svmm_parse_method_attribute(env,cli,current,size,class,attribute) \
 _svmh_parse_method_attribute (env, cli, &current, &size, class, &attribute)
 
-static jint
+svm_static jint
 _svmh_parse_method_attribute (_svmt_JNIEnv *env,
 			      _svmt_class_loader_info *class_loader_info,
 			      _svmt_u8 **pcurrent, size_t *psize,
@@ -1407,7 +1430,7 @@ _svmh_parse_class_attribute
 #define _svmm_parse_class_attribute(env,cli,current,size,class,attribute) \
 _svmh_parse_class_attribute (env, cli, &current, &size, class, &attribute)
 
-static jint
+svm_static jint
 _svmh_parse_class_attribute (_svmt_JNIEnv *env,
 			     _svmt_class_loader_info *class_loader_info,
 			     _svmt_u8 **pcurrent, size_t *psize,
@@ -1468,7 +1491,7 @@ _svmh_parse_class_file
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmh_parse_class_file (_svmt_JNIEnv *env,
 			_svmt_class_loader_info *class_loader_info,
 			size_t class_file_length,
