@@ -90,16 +90,41 @@
 #define SVM_BCI_FAILING 2
 #define SVM_BCI_SIGFPE 3
 
-#endif
+#endif /* _SABLEVM_INLINABILITY_TESTING */
 
 /* needed for Thread.interrupt() */
 #define SVM_INTERRUPT_SIGNAL SIGUSR1
 /* note: these are *flags* not values! */
+#define SVM_THREAD_NOT_INTERRUPTED 0
 #define SVM_THREAD_INTERRUPTED 1
 #define SVM_THREAD_THROW_INTERRUPTED 2
 #define SVM_THREAD_INTERRUPTIBLE_BY_SIGNAL 4
 #define SVM_THREAD_INTERRUPTIBLE_ON_FAT_LOCK 8
 
+/* needed to avoid explicit m4 in instructions.m4.c */
+#ifdef SABLEVM_GC_WRITE_BARRIER
+#define _SABLEVM_GC_WRITE_BARRIER_REFERENCE
+#endif
+
+/* min/max values of java integer types */
+#define SVM_JBYTE_MIN (-128)
+#define SVM_JBYTE_MAX   127
+
+#define SVM_JCHAR_MIN 0
+#define SVM_JCHAR_MAX 65535
+
+#define SVM_JSHORT_MIN (-32767 -1)
+#define SVM_JSHORT_MAX   32767
+
+#define SVM_JINT_MIN (-2147483647 -1)
+#define SVM_JINT_MAX   2147483647
+
+#define SVM_JLONG_MIN (((_svmt_s64) 1) << 63)
+#define SVM_JLONG_MAX ((_svmt_s64) (((_svmt_u64) SVM_JLONG_MIN) - 1))
+
+/* #define SVM_JLONG_MIN (-9223372036854775807LL -1LL)
+   #define SVM_JLONG_MAX   9223372036854775807LL */
+
 /* various constants */
 #define SVM_FRAME_NATIVE_REFS_MIN 16
 #define SVM_FRAME_NATIVE_REFS_AUTOADD_STEP 16
@@ -134,6 +159,9 @@
 #define SVM_THREAD_STATUS_NOT_RUNNING_JAVA_RESUMING_ALLOWED 3
 #define SVM_THREAD_STATUS_NOT_RUNNING_JAVA_RESUMING_DISALLOWED 4
 
+/* maximum size of a string to keep instruction name */
+#define _SABLEVM_BYTECODE_NAME_MAX_LENGTH 30
+
 /* instructions */
 #define SVM_INSTRUCTION_NOP   0
 #define SVM_INSTRUCTION_ACONST_NULL   1
