@@ -14,7 +14,7 @@ _svmf_heap_init_defaults
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_heap_init_defaults (_svmt_JavaVM *vm)
 {
   vm->heap.min_size = SVM_HEAP_DEFAULT_MIN_SIZE;
@@ -28,7 +28,7 @@ _svmf_heap_init
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_heap_init (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -52,7 +52,7 @@ _svmf_heap_init (_svmt_JNIEnv *env)
     {
       if (vm->verbose_gc)
 	{
-	  _svmf_printf (env, stdout,
+	  _svmf_printf (env, stderr,
 			"[verbose gc: allocating fixed size heap (2 * %d bytes)]\n",
 			vm->heap.min_size);
 	}
@@ -73,7 +73,7 @@ _svmf_heap_init (_svmt_JNIEnv *env)
     {
       if (vm->verbose_gc)
 	{
-	  _svmf_printf (env, stdout,
+	  _svmf_printf (env, stderr,
 			"[verbose gc: allocating initial heap (%d bytes)]\n",
 			vm->heap.min_size);
 	}
@@ -105,7 +105,7 @@ _svmf_copy_object
  * this function returns a pointer to the to_space copy (or NULL).
  */
 
-inline static _svmt_object_instance *
+inline svm_static _svmt_object_instance *
 _svmf_copy_object (_svmt_JNIEnv *env, _svmt_object_instance *obj,
 		   void **pto_space_tail)
 {
@@ -229,7 +229,8 @@ _svmf_copy_object (_svmt_JNIEnv *env, _svmt_object_instance *obj,
 
 	  memcpy (*pto_space_tail, ((char *) array) - offset, instance_size);
 	  result =
-	    (_svmt_object_instance *) (((char *) *pto_space_tail) + offset);
+	    (_svmt_object_instance *) (void *) (((char *) *pto_space_tail) +
+						offset);
 	}
       else
 	{
@@ -284,8 +285,8 @@ _svmf_copy_object (_svmt_JNIEnv *env, _svmt_object_instance *obj,
 	  memcpy (*pto_space_tail, ((char *) obj) - start_offset,
 		  instance_size);
 	  result =
-	    (_svmt_object_instance *) (((char *) *pto_space_tail) +
-				       start_offset);
+	    (_svmt_object_instance *) (void *) (((char *) *pto_space_tail) +
+						start_offset);
 	}
 	break;
 
@@ -294,8 +295,8 @@ _svmf_copy_object (_svmt_JNIEnv *env, _svmt_object_instance *obj,
 	  memcpy (*pto_space_tail, ((char *) obj) - start_offset,
 		  instance_size);
 	  result =
-	    (_svmt_object_instance *) (((char *) *pto_space_tail) +
-				       start_offset);
+	    (_svmt_object_instance *) (void *) (((char *) *pto_space_tail) +
+						start_offset);
 
 	  instance_size += SVM_ALIGNMENT;
 	  _svmm_lockword_set_hashstate (result->lockword, SVM_HASH_MOVED);
@@ -310,8 +311,8 @@ _svmf_copy_object (_svmt_JNIEnv *env, _svmt_object_instance *obj,
 	  memcpy (*pto_space_tail, ((char *) obj) - start_offset,
 		  instance_size);
 	  result =
-	    (_svmt_object_instance *) (((char *) *pto_space_tail) +
-				       start_offset);
+	    (_svmt_object_instance *) (void *) (((char *) *pto_space_tail) +
+						start_offset);
 	}
 	break;
 
@@ -412,7 +413,7 @@ _svmf_trace_native_ref_list
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_trace_native_ref_list (_svmt_JNIEnv *env, _svmt_native_ref *native_list,
 			     void **pto_space_tail)
 {
@@ -430,7 +431,7 @@ _svmf_trace_stack
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_trace_stack (_svmt_JNIEnv *env, _svmt_JNIEnv *thread,
 		   void **pto_space_tail)
 {
@@ -442,13 +443,13 @@ _svmf_trace_stack (_svmt_JNIEnv *env, _svmt_JNIEnv *thread,
     {
       if (!_svmf_is_set_flag (method->access_flags, SVM_ACC_INTERNAL))
 	{
-	  _svmt_stack_value *locals = (_svmt_stack_value *)
+	  _svmt_stack_value *locals = (_svmt_stack_value *) (void *)
 	    (((char *) frame) - method->frame_info->start_offset);
 	  _svmt_gc_map_node *parameters_gc_map = method->parameters_gc_map;
 	  jint non_parameter_ref_locals_count =
 	    method->frame_info->non_parameter_ref_locals_count;
 
-	  _svmt_stack_value *stack = (_svmt_stack_value *)
+	  _svmt_stack_value *stack = (_svmt_stack_value *) (void *)
 	    (((char *) frame) + _svmv_stack_offset);
 	  jint stack_size = frame->stack_size;
 	  _svmt_gc_map_node *stack_gc_map =
@@ -508,7 +509,7 @@ _svmf_trace_stack (_svmt_JNIEnv *env, _svmt_JNIEnv *thread,
 	    }
 	}
 
-      frame = (_svmt_stack_frame *)
+      frame = (_svmt_stack_frame *) (void *)
 	(((char *) frame) - frame->previous_offset);
       method = frame->method;
     }
@@ -523,7 +524,7 @@ _svmf_copy_gc_internal
 /* IMPORTANT: The world must be stopped prior to calling this
    function. */
 
-static jint
+svm_static jint
 _svmf_copy_gc_internal (_svmt_JNIEnv *env, size_t requested_size)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -583,7 +584,7 @@ _svmf_copy_gc_internal (_svmt_JNIEnv *env, size_t requested_size)
 
   if (vm->verbose_gc)
     {
-      _svmf_printf (env, stdout, "[verbose gc: ");
+      _svmf_printf (env, stderr, "[verbose gc: ");
       gettimeofday (&starttime, NULL);
     }
 
@@ -741,7 +742,8 @@ _svmf_copy_gc_internal (_svmt_JNIEnv *env, size_t requested_size)
 #endif
 
 	  /* skip to next object */
-	  to_space_head = (_svmt_word *) (((char *) array) + instance_size);
+	  to_space_head =
+	    (_svmt_word *) (void *) (((char *) array) + instance_size);
 
 	  continue;
 	}
@@ -782,7 +784,7 @@ _svmf_copy_gc_internal (_svmt_JNIEnv *env, size_t requested_size)
 	  }
 
 	/* skip to next object */
-	to_space_head = (_svmt_word *) (((char *) obj) + end_offset);
+	to_space_head = (_svmt_word *) (void *) (((char *) obj) + end_offset);
 
 	continue;
 
@@ -892,7 +894,7 @@ _svmf_copy_gc_internal (_svmt_JNIEnv *env, size_t requested_size)
   if (vm->verbose_gc)
     {
       gettimeofday (&endtime, NULL);
-      _svmf_printf (env, stdout,
+      _svmf_printf (env, stderr,
 		    "previously allocated %d bytes, surviving %d bytes, ",
 		    ((char *) vm->heap.alloc) - ((char *) vm->heap.start),
 		    ((char *) to_space_tail) - ((char *) to_space));
@@ -968,7 +970,7 @@ _svmf_copy_gc_internal (_svmt_JNIEnv *env, size_t requested_size)
 	  secs -= 1;
 	}
 
-      _svmf_printf (env, stdout,
+      _svmf_printf (env, stderr,
 		    "new heap is %d bytes, gc time = %ld sec %ld usec]\n",
 		    ((char *) vm->heap.end) - ((char *) vm->heap.start), secs,
 		    usecs);
@@ -1001,7 +1003,7 @@ _svmf_copy_gc
 /* IMPORTANT: The calling thread should hold the lock on
    vm->global_mutex when calling this function. */
 
-static jint
+svm_static jint
 _svmf_copy_gc_no_exception (_svmt_JNIEnv *env, size_t requested_size)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -1026,7 +1028,7 @@ _svmf_gc_new_instance
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_gc_new_instance (_svmt_JNIEnv *env, size_t instance_size,
 		       void **pinstance)
 {
