@@ -604,10 +604,10 @@ struct JNIInvokeInterface_struct
 
   /* invocation API */
 
-  JNIEXPORT jint JNICALL JNI_GetDefaultJavaVMInitArgs (void *vm_args);
-  JNIEXPORT jint JNICALL JNI_GetCreatedJavaVMs (JavaVM **vmBuf, jsize bufLen,
+  JNIIMPORT jint JNICALL JNI_GetDefaultJavaVMInitArgs (void *vm_args);
+  JNIIMPORT jint JNICALL JNI_GetCreatedJavaVMs (JavaVM **vmBuf, jsize bufLen,
 						jsize *nVMs);
-  JNIEXPORT jint JNICALL JNI_CreateJavaVM (JavaVM **pvm, void **penv,
+  JNIIMPORT jint JNICALL JNI_CreateJavaVM (JavaVM **pvm, void **penv,
 					   void *vm_args);
 
   /* library and version management */
