@@ -24,7 +24,10 @@
      ) && defined (__GNUC__))
 
 #define JNICALL
+
+/* See http://sablevm.org/bugs/14 for more information about the following */
 #define JNIEXPORT
+#define JNIIMPORT
 
 typedef unsigned char _svmt_u8;
 typedef signed char _svmt_s8;
