@@ -31,7 +31,7 @@ Java_java_lang_VMClass_isInstance
  */
 
 JNIEXPORT static jboolean JNICALL
-Java_java_lang_VMClass_isInstance (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_isInstance (JNIEnv *_env, jclass vmclass SVM_UNUSED,
 				   jclass theclass, jobject obj)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -64,7 +64,7 @@ Java_java_lang_VMClass_getModifiers
  */
 
 JNIEXPORT static jint JNICALL
-Java_java_lang_VMClass_getModifiers (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getModifiers (JNIEnv *_env, jclass vmclass SVM_UNUSED,
 				     jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -100,7 +100,8 @@ Java_java_lang_VMClass_getDeclaringClass
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClass_getDeclaringClass (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getDeclaringClass (JNIEnv *_env,
+					  jclass vmclass SVM_UNUSED,
 					  jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -193,7 +194,8 @@ Java_java_lang_VMClass_getDeclaredClasses
  */
 
 JNIEXPORT static jobjectArray JNICALL
-Java_java_lang_VMClass_getDeclaredClasses (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getDeclaredClasses (JNIEnv *_env,
+					   jclass vmclass SVM_UNUSED,
 					   jclass theclass,
 					   jboolean publicOnly)
 {
@@ -377,7 +379,7 @@ Java_java_lang_VMClass_isInterface
  */
 
 JNIEXPORT static jboolean JNICALL
-Java_java_lang_VMClass_isInterface (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_isInterface (JNIEnv *_env, jclass vmclass SVM_UNUSED,
 				    jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -417,7 +419,7 @@ Java_java_lang_VMClass_getSuperclass
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClass_getSuperclass (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getSuperclass (JNIEnv *_env, jclass vmclass SVM_UNUSED,
 				      jclass theclass)
 {
 
@@ -479,7 +481,7 @@ Java_java_lang_VMClass_getInterfaces
  */
 
 JNIEXPORT static jobjectArray JNICALL
-Java_java_lang_VMClass_getInterfaces (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getInterfaces (JNIEnv *_env, jclass vmclass SVM_UNUSED,
 				      jclass theclass)
 {
 
@@ -494,19 +496,6 @@ Java_java_lang_VMClass_getInterfaces (JNIEnv *_env, jclass vmclass,
     /* The type Class[] */
     static _svmt_array_info *class_array_info = NULL;
 
-    jboolean monitor_acquired = JNI_FALSE;
-
-    if (_svmf_enter_object_monitor
-	(env,
-	 *(vm->class_loading.boot_loader.classes.virtualmachine->
-	   class_instance)) != JNI_OK)
-      {
-	goto end;
-      }
-
-    monitor_acquired = JNI_TRUE;
-
-
     {
       /* Load/link/init type Class[] if necessary. */
 
@@ -573,32 +562,25 @@ Java_java_lang_VMClass_getInterfaces (JNIEnv *_env, jclass vmclass,
 	  if (_svmf_set_reference_array_element_no_exception
 	      (env, *interfaces_array, i, *interface) != JNI_OK)
 	    {
+	      _svmm_release_jni_frame_native_local_array (interfaces_array);
 	      goto end;
 	    }
 	}
     }
 
-  end:
-    if (monitor_acquired)
-      {
-	monitor_acquired = JNI_FALSE;
-
-	if (_svmf_exit_object_monitor
-	    (env,
-	     *(vm->class_loading.boot_loader.classes.virtualmachine->
-	       class_instance)) != JNI_OK)
-	  {
-	    goto end;
-	  }
-      }
   }
 
-
+end:
   _svmf_stopping_java (env);
 
   return interfaces_array;
 }
 
+/*
+----------------------------------------------------------------------
+Java_java_lang_VMClass_getDeclaredFields
+----------------------------------------------------------------------
+*/
 
 /*
  * Class:     java_lang_VMClass
@@ -607,227 +589,248 @@ Java_java_lang_VMClass_getInterfaces (JNIEnv *_env, jclass vmclass,
  */
 
 JNIEXPORT static jobjectArray JNICALL
-Java_java_lang_VMClass_getDeclaredFields (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getDeclaredFields (JNIEnv *_env,
+					  jclass vmclass SVM_UNUSED,
 					  jclass theclass,
 					  jboolean public_only)
 {
-
+  /* pointer to current environment (or thread-specific) structure; */
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
-  /* Array that will contain the fields. */
+
+  /* array reference that contains the return value */
   jobjectArray fields_array = NULL;
 
+  /* disable concurrent garbage collection, stopping the world if
+     garbage collection is in progress */
   _svmf_resuming_java (env);
 
   {
     _svmt_JavaVM *vm = env->vm;
-
-    /* The type Field[] */
-    static _svmt_array_info *field_array_info = NULL;
+    jboolean monitor_acquired = JNI_FALSE;
 
     /* The type Field */
     _svmt_class_info *field_class_info;
 
-    jboolean monitor_acquired = JNI_FALSE;
+    /* The type Field[] */
+    _svmt_array_info *field_array_info;
 
-    if (_svmf_enter_object_monitor
-	(env,
-	 *(vm->class_loading.boot_loader.classes.virtualmachine->
-	   class_instance)) != JNI_OK)
+    /* The class we are looking at. */
+    _svmt_class_info *class_info;
+
+    /* get classes loaded and linked at bootstrap */
+    field_class_info = vm->class_loading.boot_loader.classes.jlrfield;
+    field_array_info = vm->class_loading.boot_loader.classes.jlrfield_array;
+
+    /* unwrap the pointer to the _svmt_class_info of the class which
+       declares the fields we want to retrieve */
+    class_info =
+      _svmf_cast_class (_svmf_unwrap_class_instance (env, theclass));
+
+    /* a class must have been prepared before a Class instance is
+       associated with it.  Assert that this is true here, in debug
+       mode, to detect bugs in the VM. */
+    assert (_svmf_is_set_flag (class_info->state, SVM_TYPE_STATE_PREPARED));
+
+    /* we have to protect against concurrent creation of
+       java.lang.reflect.Field instances for the same fields of a
+       class on multiple threads */
+    assert (*(class_info->class_instance) == *(theclass));
+    if (_svmf_enter_object_monitor (env, *(theclass)) != JNI_OK)
       {
 	goto end;
       }
 
     monitor_acquired = JNI_TRUE;
 
-
-
-
-
-    field_class_info = vm->class_loading.boot_loader.classes.jlrfield;
-
-
-    /* Load/link/init type Field[]. */
-    if (field_array_info == NULL)
-      {
-
-
-	/* creates type Field[] */
-	if (_svmm_create_array
-	    (env,
-	     field_class_info->class_loader_info,
-	     field_class_info->array_type_name, field_array_info) != JNI_OK)
-	  {
-	    goto end;
-	  }
-
-	/* link the type Field[] */
-	if (_svmf_link_array (env, field_array_info) != JNI_OK)
-	  {
-	    goto end;
-	  }
-
-      }
-
-
     {
-      /* The class we are looking at. */
-      _svmt_class_info *class_info;
-
-      class_info =
-	_svmf_cast_class (_svmf_unwrap_class_instance (env, theclass));
-
-
-      /* Need to link class - to fill in field_info */
-      if (_svmf_link_class (env, class_info) != JNI_OK)
-	{
-	  goto end;
-	}
-
+      jint i;
+      _svmt_field_info *fields = class_info->fields;
 
+      /* create array of correct size */
       {
-	jint i;
-	_svmt_field_info *fields;
+	jint fields_count = 0;
+
+	if (public_only)
+	  {
+	    /* need to count them first */
+	    for (i = 0; i < class_info->fields_count; i++)
+	      {
+		_svmt_field_info *field;
+		field = &fields[i];
+		if (_svmf_is_set_flag (field->access_flags, SVM_ACC_PUBLIC))
+		  {
+		    fields_count++;
+		  }
+	      }
+	  }
+	else
+	  {
+	    fields_count = class_info->fields_count;
+	  }
 
 
-	fields = class_info->fields;
+	/* as this reference will be returned to the JNI caller
+	   function, _svmf_get_jni_frame_native_local_array must be
+	   used to obtain a JNI local reference from the current JNI
+	   local reference frame.  */
+	fields_array = _svmf_get_jni_frame_native_local_array (env);
 
-	/* create array of correct size */
-	{
-	  jint fields_count;
-	  if (public_only)
-	    {
-	      /* need to count them first */
-	      fields_count = 0;
-	      for (i = 0; i < class_info->fields_count; i++)
-		{
-		  _svmt_field_info *field;
-		  field = &fields[i];
-		  if (_svmf_is_set_flag (field->access_flags, SVM_ACC_PUBLIC))
-		    {
-		      fields_count++;
-		    }
-		}
-	    }
-	  else
-	    {
-	      fields_count = class_info->fields_count;
-	    }
-
-	  /* create an array */
-	  fields_array = _svmf_get_jni_frame_native_local_array (env);
-	  if (_svmm_new_array_instance
-	      (env, field_array_info,
-	       class_info->fields_count, *fields_array) != JNI_OK)
-	    {
-	      goto end;
-	    }
-	}
-
-	{
-	  jint array_index = 0;
-	  for (i = 0; i < class_info->fields_count; i++)
-	    {
-	      _svmt_field_info *field_info;
-	      jobject fieldObject;
-
-	      field_info = &fields[i];
-
-	      if (public_only)
-		{
-		  if (!_svmf_is_set_flag
-		      (field_info->access_flags, SVM_ACC_PUBLIC))
-		    {
-		      continue;
-		    }
-		}
-
-	      /* create Field object, if none */
-	      if (field_info->reflection_instance == NULL)
-		{
-
-		  jbyteArray wrapper;
-
-
-		  /* wrap it */
-		  wrapper = _svmf_get_jni_frame_native_local_array (env);
-		  if (_svmm_local_wrap_pointer (env, field_info, wrapper) !=
-		      JNI_OK)
-		    {
-		      goto end;
-		    }
-
-		  fieldObject = _svmf_get_jni_frame_native_local (env);
-		  if (_svmm_new_object_instance
-		      (env,
-		       vm->class_loading.boot_loader.classes.jlrfield,
-		       *fieldObject) != JNI_OK)
-		    {
-		      goto end;
-		    }
-
-		  if (_svmm_invoke_nonvirtual_jlrfield_init
-		      (env, fieldObject,
-		       _svmf_cast_jobject (wrapper)) != JNI_OK)
-		    {
-		      goto end;
-		    }
-
-		  if (_svmm_new_native_global
-		      (env, field_info->reflection_instance) != JNI_OK)
-		    {
-		      goto end;
-		    }
-
-		  /* Cache the resulting object */
-		  *(field_info->reflection_instance) = *fieldObject;
-
-		  /* release native refs */
-		  _svmm_release_jni_frame_native_local (fieldObject);
-		  _svmm_release_jni_frame_native_local_array (wrapper);
-
-		}
-
-	      /* add to array */
-	      if (_svmf_set_reference_array_element_no_exception
-		  (env, *fields_array, array_index++,
-		   *(field_info->reflection_instance)) != JNI_OK)
-		{
-		  goto end;
-		}
-
-
-	    }
-
-
-	}
+	/* create the Field[] array of the appropriate size */
+	if (_svmm_new_array_instance
+	    (env, field_array_info, fields_count, *fields_array) != JNI_OK)
+	  {
+	    goto end;
+	  }
       }
 
+      {
+	jint array_index = 0;
 
+	for (i = 0; i < class_info->fields_count; i++)
+	  {
+	    _svmt_field_info *field_info;
+	    jobject fieldObject;
 
+	    field_info = &fields[i];
 
+	    if (public_only)
+	      {
+		if (!_svmf_is_set_flag
+		    (field_info->access_flags, SVM_ACC_PUBLIC))
+		  {
+		    continue;
+		  }
+	      }
+
+	    /* Create a Field object, if none already exists for this
+	       field.  The monitor above was acquired specifically to
+	       prevent this code from executing concurrently on two
+	       thread.  We want to avoid creating more than one Field
+	       instance for this field (which could happen if we did
+	       not use synchronization). */
+	    if (field_info->reflection_instance == NULL)
+	      {
+		jbyteArray wrapper;
+
+		/* Wrap the pointer to this _svmt_field_info into a
+		   byte array, and store the pointer to that array
+		   into a local reference.
+
+		   A local reference is a reference which holds a
+		   "pointer" to an object which can be moved by the
+		   garbage collector.  Note that garbage collection
+		   can obviously happen during the call to
+		   _svmm_new_object_instance() below.  As the garbage
+		   collector can move objects around, the use of a
+		   local reference is necessary so that the code that
+		   follows the call still refers to the (possibly
+		   moved) wrapper array.
+
+		   The difference between local and a global
+		   references is that local references are obtained
+		   and released without any synchronization.  This
+		   has 2 consequences:
+
+		   (a) it is faster, but 
+
+		   (b) it is unsafe to share local references with
+		   other threads.
+
+		   A local reference is usually used by the VM to
+		   temporarily store an object references in it.
+		   Unlike a JNI reference returned to the caller
+		   function, a local reference must be freed by the
+		   VM before the end of the current function. */
+		if (_svmm_local_wrap_pointer (env, field_info, wrapper) !=
+		    JNI_OK)
+		  {
+		    _svmm_release_jni_frame_native_local_array (fields_array);
+		    goto end;
+		  }
+
+		if (_svmm_new_native_local (env, fieldObject) != JNI_OK)
+		  {
+		    _svmm_free_native_local_array (env, wrapper);
+		    _svmm_release_jni_frame_native_local_array (fields_array);
+		    goto end;
+		  }
+
+		/* create the Field instance */
+		if (_svmm_new_object_instance
+		    (env,
+		     vm->class_loading.boot_loader.classes.jlrfield,
+		     *fieldObject) != JNI_OK)
+		  {
+		    _svmm_free_native_local (env, fieldObject);
+		    _svmm_free_native_local_array (env, wrapper);
+		    _svmm_release_jni_frame_native_local_array (fields_array);
+		    goto end;
+		  }
+
+		/* call the constructor of the Field instance */
+		if (_svmm_invoke_nonvirtual_jlrfield_init
+		    (env, fieldObject,
+		     _svmf_cast_jobject (wrapper)) != JNI_OK)
+		  {
+		    _svmm_free_native_local (env, fieldObject);
+		    _svmm_free_native_local_array (env, wrapper);
+		    _svmm_release_jni_frame_native_local_array (fields_array);
+		    goto end;
+		  }
+
+		/* create a global reference to permanently store a
+		   pointer to the newly created Field instance in the
+		   _svmt_field_info structure. */
+
+		if (_svmm_new_native_global
+		    (env, field_info->reflection_instance) != JNI_OK)
+		  {
+		    _svmm_free_native_local (env, fieldObject);
+		    _svmm_free_native_local_array (env, wrapper);
+		    _svmm_release_jni_frame_native_local_array (fields_array);
+		    goto end;
+		  }
+
+		/* Store the Field reference. */
+		*(field_info->reflection_instance) = *fieldObject;
+
+		/* release local native refs, now that we're done with
+		   them.  Actually, we're in a loop, so if we don't do
+		   it, we will leak native references. */
+		_svmm_free_native_local (env, fieldObject);
+		_svmm_free_native_local_array (env, wrapper);
+	      }
+
+	    /* Copy the permanently stored the reference into the
+	       Field[] array. */
+	    if (_svmf_set_reference_array_element_no_exception
+		(env, *fields_array, array_index++,
+		 *(field_info->reflection_instance)) != JNI_OK)
+	      {
+		_svmm_fatal_error ("impossible control flow");
+	      }
+	  }
+      }
     }
 
-
   end:
     if (monitor_acquired)
       {
 	monitor_acquired = JNI_FALSE;
 
-	if (_svmf_exit_object_monitor
-	    (env,
-	     *(vm->class_loading.boot_loader.classes.virtualmachine->
-	       class_instance)) != JNI_OK)
+	if (_svmf_exit_object_monitor (env, *(theclass)) != JNI_OK)
 	  {
 	    goto end;
 	  }
       }
   }
 
-
+  /* returning to foreing JNI code.  As we don't know when this code
+     will call back (or return to) the VM, we must enable concurrent
+     garbage collection.  */
   _svmf_stopping_java (env);
 
   return fields_array;
-
 }
 
 
@@ -867,19 +870,13 @@ getDeclaredMethodsConstructors_helper (_svmt_JNIEnv *env, jclass theclass,
 
     jboolean monitor_acquired = JNI_FALSE;
 
-    if (_svmf_enter_object_monitor
-	(env,
-	 *(vm->class_loading.boot_loader.classes.virtualmachine->
-	   class_instance)) != JNI_OK)
+    if (_svmf_enter_object_monitor (env, *(theclass)) != JNI_OK)
       {
 	goto end;
       }
 
     monitor_acquired = JNI_TRUE;
 
-
-
-
     {
       /* Load/link/init type Method[] (or Constructor[]) if necessary. */
 
@@ -891,7 +888,6 @@ getDeclaredMethodsConstructors_helper (_svmt_JNIEnv *env, jclass theclass,
 	    vm->class_loading.boot_loader.classes.jlrmethod :
 	    vm->class_loading.boot_loader.classes.jlrconstructor;
 
-
 	  /* creates type Method[] or Constructor[] */
 	  if (_svmm_create_array
 	      (env,
@@ -1101,10 +1097,7 @@ getDeclaredMethodsConstructors_helper (_svmt_JNIEnv *env, jclass theclass,
       {
 	monitor_acquired = JNI_FALSE;
 
-	if (_svmf_exit_object_monitor
-	    (env,
-	     *(vm->class_loading.boot_loader.classes.virtualmachine->
-	       class_instance)) != JNI_OK)
+	if (_svmf_exit_object_monitor (env, *(theclass)) != JNI_OK)
 	  {
 	    goto end;
 	  }
@@ -1129,7 +1122,8 @@ Java_java_lang_VMClass_getDeclaredMethods
  */
 
 JNIEXPORT static jobjectArray JNICALL
-Java_java_lang_VMClass_getDeclaredMethods (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getDeclaredMethods (JNIEnv *_env,
+					   jclass vmclass SVM_UNUSED,
 					   jclass theclass,
 					   jboolean publicOnly)
 {
@@ -1153,7 +1147,8 @@ Java_java_lang_VMClass_getDeclaredConstructors
  */
 
 JNIEXPORT static jobjectArray JNICALL
-Java_java_lang_VMClass_getDeclaredConstructors (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getDeclaredConstructors (JNIEnv *_env,
+						jclass vmclass SVM_UNUSED,
 						jclass theclass,
 						jboolean publicOnly)
 {
@@ -1177,7 +1172,8 @@ Java_java_lang_VMClass_isAssignableFrom
  */
 
 JNIEXPORT static jboolean JNICALL
-Java_java_lang_VMClass_isAssignableFrom (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_isAssignableFrom (JNIEnv *_env,
+					 jclass vmclass SVM_UNUSED,
 					 jclass theclass, jclass c)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -1210,7 +1206,8 @@ Java_java_lang_VMClass_getClassLoader
  */
 
 JNIEXPORT static jobject JNICALL
-Java_java_lang_VMClass_getClassLoader (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_getClassLoader (JNIEnv *_env,
+				       jclass vmclass SVM_UNUSED,
 				       jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -1247,7 +1244,8 @@ Java_java_lang_VMClass_getName
  */
 
 JNIEXPORT static jstring JNICALL
-Java_java_lang_VMClass_getName (JNIEnv *_env, jclass vmclass, jclass theclass)
+Java_java_lang_VMClass_getName (JNIEnv *_env, jclass vmclass SVM_UNUSED,
+				jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jstring name;
@@ -1280,7 +1278,7 @@ Java_java_lang_VMClass_isInitialized
  */
 
 JNIEXPORT static jboolean JNICALL
-Java_java_lang_VMClass_isInitialized (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_isInitialized (JNIEnv *_env, jclass vmclass SVM_UNUSED,
 				      jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -1289,7 +1287,6 @@ Java_java_lang_VMClass_isInitialized (JNIEnv *_env, jclass vmclass,
   _svmf_resuming_java (env);
 
   {
-    _svmt_JavaVM *vm = env->vm;
     _svmt_class_info *class =
       (_svmt_class_info *) _svmf_unwrap_class_instance (env, theclass);
 
@@ -1316,7 +1313,8 @@ Java_java_lang_VMClass_setInitialized
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_VMClass_setInitialized (JNIEnv *_env, jclass vmclass,
+Java_java_lang_VMClass_setInitialized (JNIEnv *_env,
+				       jclass vmclass SVM_UNUSED,
 				       jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -1324,7 +1322,6 @@ Java_java_lang_VMClass_setInitialized (JNIEnv *_env, jclass vmclass,
   _svmf_resuming_java (env);
 
   {
-    _svmt_JavaVM *vm = env->vm;
     _svmt_class_info *class =
       (_svmt_class_info *) _svmf_unwrap_class_instance (env, theclass);
 
@@ -1347,14 +1344,14 @@ Java_java_lang_VMClass_step7
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_VMClass_step7 (JNIEnv *_env, jclass vmclass, jclass theclass)
+Java_java_lang_VMClass_step7 (JNIEnv *_env, jclass vmclass SVM_UNUSED,
+			      jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
   _svmf_resuming_java (env);
 
   {
-    _svmt_JavaVM *vm = env->vm;
     _svmt_class_info *class =
       (_svmt_class_info *) _svmf_unwrap_class_instance (env, theclass);
 
@@ -1391,14 +1388,14 @@ Java_java_lang_VMClass_step8
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_VMClass_step8 (JNIEnv *_env, jclass vmclass, jclass theclass)
+Java_java_lang_VMClass_step8 (JNIEnv *_env, jclass vmclass SVM_UNUSED,
+			      jclass theclass)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
   _svmf_resuming_java (env);
 
   {
-    _svmt_JavaVM *vm = env->vm;
     _svmt_class_info *class =
       (_svmt_class_info *) _svmf_unwrap_class_instance (env, theclass);
     _svmt_method_info *clinit;
@@ -1427,7 +1424,7 @@ end:
 
 
 JNIEXPORT static void JNICALL
-Java_java_lang_VMClass_throwException (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClass_throwException (JNIEnv *_env, jclass _class SVM_UNUSED,
 				       jthrowable exception)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
