@@ -8,36 +8,49 @@
 
 /*
 ----------------------------------------------------------------------
-Java_java_lang_VMClassLoader_getClassLoadingLock
+Java_java_lang_VMClassLoader_nativeGetResource
 ----------------------------------------------------------------------
 */
 
 /*
  * Class:     java_lang_VMClassLoader
- * Method:    getClassLoadingLock
- * Signature: ()Ljava/lang/Object;
+ * Method:    nativeGetResource
+ * Signature: (Ljava/lang/String)Ljava/lang/String;
  */
 
-JNIEXPORT static jobject JNICALL
-Java_java_lang_VMClassLoader_getClassLoadingLock (JNIEnv *_env, jclass _class)
+JNIEXPORT static jstring JNICALL
+Java_java_lang_VMClassLoader_nativeGetResource (JNIEnv *_env,
+						jclass _class SVM_UNUSED,
+						jstring _name)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
-  jobject lock;
+  jstring result = NULL;
+  char *res_url, *name;
 
   _svmf_resuming_java (env);
 
   {
-    _svmt_JavaVM *vm = env->vm;
+    if (_svmm_galloc_utf_chars (env, _name, name) != JNI_OK)
+      {
+	goto end;
+      }
 
-    lock = _svmf_get_jni_frame_native_local (env);
-    *lock =
-      *(vm->class_loading.boot_loader.classes.virtualmachine->class_instance);
+    res_url = _svmf_bootcl_get_resource_url (env, name);
+
+    if (res_url != NULL)
+      {
+	result = _svmf_get_jni_frame_native_local (env);
+	_svmf_get_string (env, res_url, result);
+	_svmf_free (res_url);
+      }
+
+    _svmf_free (name);
   }
 
 end:
   _svmf_stopping_java (env);
 
-  return lock;
+  return result;
 }
 
 /*
@@ -54,7 +67,8 @@ Java_java_lang_VMClassLoader_newClassLoaderVmData
 
 JNIEXPORT static jbyteArray JNICALL
 Java_java_lang_VMClassLoader_newClassLoaderVmData (JNIEnv *_env,
-						   jclass _class, jobject cl)
+						   jclass _class SVM_UNUSED,
+						   jobject cl)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jbyteArray vmData = NULL;
@@ -130,7 +144,8 @@ Java_java_lang_VMClassLoader_getPrimitiveArray
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClassLoader_getPrimitiveArray (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClassLoader_getPrimitiveArray (JNIEnv *_env,
+						jclass _class SVM_UNUSED,
 						jchar element)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -262,7 +277,7 @@ Java_java_lang_VMClassLoader_getArray
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClassLoader_getArray (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClassLoader_getArray (JNIEnv *_env, jclass _class SVM_UNUSED,
 				       jclass _element)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -271,7 +286,10 @@ Java_java_lang_VMClassLoader_getArray (JNIEnv *_env, jclass _class,
   _svmf_resuming_java (env);
 
   {
+#ifndef NDEBUG
     _svmt_JavaVM *vm = env->vm;
+#endif
+
     _svmt_type_info *element = _svmf_unwrap_class_instance (env, _element);
     _svmt_array_info *array_type;
 
@@ -308,7 +326,8 @@ Java_java_lang_VMClassLoader_nativeDefineArray
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClassLoader_nativeDefineArray (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClassLoader_nativeDefineArray (JNIEnv *_env,
+						jclass _class SVM_UNUSED,
 						jbyteArray cl, jstring name,
 						jclass _element)
 {
@@ -318,7 +337,6 @@ Java_java_lang_VMClassLoader_nativeDefineArray (JNIEnv *_env, jclass _class,
   _svmf_resuming_java (env);
 
   {
-    _svmt_JavaVM *vm = env->vm;
     _svmt_class_loader_info *class_loader_info = _svmf_unwrap_pointer (*cl);
     _svmt_type_info *element = _svmf_unwrap_class_instance (env, _element);
     _svmt_array_info *array;
@@ -436,10 +454,12 @@ Java_java_lang_VMClassLoader_nativeDefineClass
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClassLoader_nativeDefineClass (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClassLoader_nativeDefineClass (JNIEnv *_env,
+						jclass _class SVM_UNUSED,
 						jbyteArray cl, jstring name,
 						jbyteArray data, jint offset,
-						jint len)
+						jint len,
+						jobject protection_domain)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jclass class_instance = NULL;
@@ -494,6 +514,8 @@ Java_java_lang_VMClassLoader_nativeDefineClass (JNIEnv *_env, jclass _class,
       _svmm_gfree_utf_chars (name_utf);
     }
 
+    class->protection_domain = protection_domain;
+
     if (_svmf_resolve_super_class (env, class) != JNI_OK ||
 	_svmf_resolve_super_interfaces (env, class) != JNI_OK)
       {
@@ -505,7 +527,9 @@ Java_java_lang_VMClassLoader_nativeDefineClass (JNIEnv *_env, jclass _class,
 	goto end;
       }
 
-    if (_svmf_new_class (env, _svmf_cast_type_class (class)) != JNI_OK)
+    if (_svmf_new_class_pd
+	(env, _svmf_cast_type_class (class),
+	 class->protection_domain) != JNI_OK)
       {
 	goto end;
       }
@@ -533,7 +557,8 @@ Java_java_lang_VMClassLoader_linkClass
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_VMClassLoader_linkClass (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClassLoader_linkClass (JNIEnv *_env,
+					jclass _class SVM_UNUSED,
 					jclass class)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -566,7 +591,8 @@ Java_java_lang_VMClassLoader_initializeClass
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_VMClassLoader_initializeClass (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClassLoader_initializeClass (JNIEnv *_env,
+					      jclass _class SVM_UNUSED,
 					      jclass class)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -609,8 +635,10 @@ Java_java_lang_VMClassLoader_nativeLoadClass
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClassLoader_nativeLoadClass (JNIEnv *_env, jclass _class,
-					      jstring _name, jboolean resolve)
+Java_java_lang_VMClassLoader_nativeLoadClass (JNIEnv *_env,
+					      jclass _class SVM_UNUSED,
+					      jstring _name,
+					      jboolean resolve SVM_UNUSED)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jclass class = NULL;
@@ -619,8 +647,6 @@ Java_java_lang_VMClassLoader_nativeLoadClass (JNIEnv *_env, jclass _class,
 
   {
     _svmt_JavaVM *vm = env->vm;
-    _svmt_stack_frame *frame = env->stack.current_frame;
-    _svmt_method_info *caller = NULL;
     _svmt_class_loader_info *class_loader_info =
       vm->class_loading.boot_loader.class_loader_info;
     char *name;
@@ -679,7 +705,8 @@ Java_java_lang_VMClassLoader_nativeCreateArray
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClassLoader_nativeCreateArray (JNIEnv *_env, jclass _class,
+Java_java_lang_VMClassLoader_nativeCreateArray (JNIEnv *_env,
+						jclass _class SVM_UNUSED,
 						jstring _name,
 						jboolean resolve)
 {
@@ -690,8 +717,6 @@ Java_java_lang_VMClassLoader_nativeCreateArray (JNIEnv *_env, jclass _class,
 
   {
     _svmt_JavaVM *vm = env->vm;
-    _svmt_stack_frame *frame = env->stack.current_frame;
-    _svmt_method_info *caller = NULL;
     _svmt_class_loader_info *class_loader_info =
       vm->class_loading.boot_loader.class_loader_info;
     char *name;
@@ -746,7 +771,8 @@ Java_java_lang_VMClassLoader_getPrimitiveClass
  */
 
 JNIEXPORT static jclass JNICALL
-Java_java_lang_VMClassLoader_getPrimitiveClass (JNIEnv *_env, jclass class,
+Java_java_lang_VMClassLoader_getPrimitiveClass (JNIEnv *_env,
+						jclass class SVM_UNUSED,
 						jchar t)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -818,6 +844,7 @@ Java_java_lang_VMClassLoader_getPrimitiveClass (JNIEnv *_env, jclass class,
       default:
 	{
 	  _svmf_error_InternalError (env);
+	  goto end;
 	}
 	break;
       }
