@@ -29,7 +29,8 @@ Java_java_lang_reflect_Field_nativeGetDeclaringClass
  */
 JNIEXPORT static jclass JNICALL
 Java_java_lang_reflect_Field_nativeGetDeclaringClass (JNIEnv *_env,
-						      jclass _class,
+						      jclass _class
+						      SVM_UNUSED,
 						      jbyteArray vmData)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -62,7 +63,8 @@ Java_java_lang_reflect_Field_nativeGetModifiers
  * Signature: ([B)I
  */
 JNIEXPORT static jint JNICALL
-Java_java_lang_reflect_Field_nativeGetModifiers (JNIEnv *_env, jobject _class,
+Java_java_lang_reflect_Field_nativeGetModifiers (JNIEnv *_env,
+						 jobject _class SVM_UNUSED,
 						 jbyteArray vmData)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -101,7 +103,8 @@ Java_java_lang_reflect_Field_nativeGetType
  * Signature: ([B)Ljava/lang/Class;
  */
 JNIEXPORT static jclass JNICALL
-Java_java_lang_reflect_Field_nativeGetType (JNIEnv *_env, jclass _class,
+Java_java_lang_reflect_Field_nativeGetType (JNIEnv *_env,
+					    jclass _class SVM_UNUSED,
 					    jbyteArray vmData)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -113,139 +116,119 @@ Java_java_lang_reflect_Field_nativeGetType (JNIEnv *_env, jclass _class,
     _svmt_JavaVM *vm = env->vm;
     _svmt_field_info *field = _svmf_unwrap_pointer (*vmData);
 
-    class = _svmf_get_jni_frame_native_local (env);
     switch (field->type)
       {
       case SVM_TYPE_BOOLEAN:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.boolean_type);
 	break;
       case SVM_TYPE_BYTE:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.byte_type);
 	break;
       case SVM_TYPE_SHORT:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.short_type);
 	break;
       case SVM_TYPE_CHAR:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.char_type);
 	break;
       case SVM_TYPE_INT:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.int_type);
 	break;
       case SVM_TYPE_LONG:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.long_type);
 	break;
       case SVM_TYPE_FLOAT:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.float_type);
 	break;
       case SVM_TYPE_DOUBLE:
+	class = _svmf_get_jni_frame_native_local (env);
 	*class = *(vm->class_loading.boot_loader.instances.double_type);
 	break;
       case SVM_TYPE_REFERENCE:
 	{
-	  jboolean monitor_acquired = JNI_FALSE;
+	  /* Based on JNI FindClass */
+	  char *descriptor;
+	  char *name = NULL;
+	  jboolean free_name;
+	  _svmt_class_loader_info *class_loader_info;
 
-	  if (_svmf_enter_object_monitor
-	      (env,
-	       *(vm->class_loading.boot_loader.classes.virtualmachine->
-		 class_instance)) != JNI_OK)
+	  descriptor = DREF (field->descriptor, value);
+
+	  assert (descriptor != NULL);
+	  assert (descriptor[0] == 'L' || descriptor[0] == '[');
+
+	  if (descriptor[0] == 'L')
 	    {
-	      goto end;
+	      /*
+	       * Convert name for non-array types.
+	       *
+	       * Ljava/lang/String; -> java/lang/String
+	       *
+	       */
+	      size_t len;
+	      len = strlen (descriptor) - 2;
+	      name = _svmf_malloc (len + 1);
+	      if (name == NULL)
+		{
+		  _svmf_error_OutOfMemoryError (env);
+		  goto end;
+		}
+	      strncpy (name, descriptor + 1, len);
+	      name[len] = '\0';
+	      free_name = JNI_TRUE;
+	    }
+	  else
+	    {
+	      name = descriptor;
+	      free_name = JNI_FALSE;
 	    }
 
+	  class_loader_info = field->class_info->class_loader_info;
+
+
+	  /* load, link, and initialize type */
 	  {
-	    /* Based on JNI FindClass */
-	    char *descriptor;
-	    char *name = NULL;
-	    jboolean free_name;
-	    _svmt_class_loader_info *class_loader_info;
+	    _svmt_type_info *type;
 
-	    descriptor = DREF (field->descriptor, value);
-
-	    assert (descriptor != NULL);
-	    assert (descriptor[0] == 'L' || descriptor[0] == '[');
-
-	    if (descriptor[0] == 'L')
+	    if (_svmm_create_type (env, class_loader_info, name, type) !=
+		JNI_OK)
 	      {
-		/*
-		 * Convert name for non-array types.
-		 *
-		 * Ljava/lang/String; -> java/lang/String
-		 *
-		 */
-		size_t len;
-		len = strlen (descriptor) - 2;
-		name = _svmf_malloc (len + 1);
-		if (name == NULL)
+		if (free_name)
+		  {
+		    _svmf_free (name);
+		  }
+
+		goto end;
+	      }
+
+	    if (free_name)
+	      {
+		_svmf_free (name);
+	      }
+
+	    if (_svmf_link_type (env, type) != JNI_OK)
+	      {
+		goto end;
+	      }
+
+	    if (!(type->is_array))
+	      {
+		if (_svmf_class_initialization
+		    (env, _svmf_cast_class (type)) != JNI_OK)
 		  {
-		    _svmf_error_OutOfMemoryError (env);
 		    goto end;
 		  }
-		strncpy (name, descriptor + 1, len);
-		name[len] = '\0';
-		free_name = JNI_TRUE;
-	      }
-	    else
-	      {
-		name = descriptor;
-		free_name = JNI_FALSE;
 	      }
 
-	    class_loader_info = field->class_info->class_loader_info;
-
-
-	    /* load, link, and initialize type */
-	    {
-	      _svmt_type_info *type;
-
-	      if (_svmm_create_type (env, class_loader_info, name, type) !=
-		  JNI_OK)
-		{
-		  if (free_name)
-		    {
-		      _svmf_free (name);
-		    }
-		  goto end;
-		}
-
-	      if (free_name)
-		{
-		  _svmf_free (name);
-		}
-
-	      if (_svmf_link_type (env, type) != JNI_OK)
-		{
-		  goto end;
-		}
-
-	      if (!(type->is_array))
-		{
-		  if (_svmf_class_initialization
-		      (env, _svmf_cast_class (type)) != JNI_OK)
-		    {
-		      goto end;
-		    }
-		}
-
-	      *class = *(type->class_instance);
-
-	    }
-
+	    class = _svmf_get_jni_frame_native_local (env);
+	    *class = *(type->class_instance);
 	  }
-
-	end:
-	  if (monitor_acquired)
-	    {
-	      monitor_acquired = JNI_FALSE;
-
-	      if (_svmf_exit_object_monitor
-		  (env,
-		   *(vm->class_loading.boot_loader.classes.virtualmachine->
-		     class_instance)) != JNI_OK)
-		{
-		  goto end;
-		}
-	    }
-
 	}
 	break;
 
@@ -253,9 +236,9 @@ Java_java_lang_reflect_Field_nativeGetType (JNIEnv *_env, jclass _class,
 	_svmm_fatal_error ("impossible control flow");
 	break;
       }
-
   }
 
+end:
   _svmf_stopping_java (env);
 
   return class;
@@ -277,7 +260,8 @@ Java_java_lang_reflect_Field_nativeGetName
  */
 
 JNIEXPORT static jstring JNICALL
-Java_java_lang_reflect_Field_nativeGetName (JNIEnv *_env, jclass _class,
+Java_java_lang_reflect_Field_nativeGetName (JNIEnv *_env,
+					    jclass _class SVM_UNUSED,
 					    jbyteArray vmData)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
@@ -310,7 +294,8 @@ Java_java_lang_reflect_Field_nativeGetReference
  * Signature: ([BLjava/lang/Object;)Ljava/lang/Object;
  */
 JNIEXPORT static jobject JNICALL
-Java_java_lang_reflect_Field_nativeGetReference (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetReference (JNIEnv *_env,
+						 jobject this SVM_UNUSED,
 						 jbyteArray vmData,
 						 jobject instance)
 {
@@ -358,7 +343,8 @@ Java_java_lang_reflect_Field_nativeGetBoolean
  * Signature: ([BLjava/lang/Object;)Z
  */
 JNIEXPORT static jboolean JNICALL
-Java_java_lang_reflect_Field_nativeGetBoolean (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetBoolean (JNIEnv *_env,
+					       jobject this SVM_UNUSED,
 					       jbyteArray vmData,
 					       jobject instance)
 {
@@ -403,7 +389,8 @@ Java_java_lang_reflect_Field_nativeGetByte
  * Signature: ([BLjava/lang/Object;)B
  */
 JNIEXPORT static jbyte JNICALL
-Java_java_lang_reflect_Field_nativeGetByte (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetByte (JNIEnv *_env,
+					    jobject this SVM_UNUSED,
 					    jbyteArray vmData,
 					    jobject instance)
 {
@@ -449,7 +436,8 @@ Java_java_lang_reflect_Field_nativeGetChar
  * Signature: ([BLjava/lang/Object;)C
  */
 JNIEXPORT static jchar JNICALL
-Java_java_lang_reflect_Field_nativeGetChar (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetChar (JNIEnv *_env,
+					    jobject this SVM_UNUSED,
 					    jbyteArray vmData,
 					    jobject instance)
 {
@@ -494,7 +482,8 @@ Java_java_lang_reflect_Field_nativeGetShort
  * Signature: ([BLjava/lang/Object;)S
  */
 JNIEXPORT static jshort JNICALL
-Java_java_lang_reflect_Field_nativeGetShort (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetShort (JNIEnv *_env,
+					     jobject this SVM_UNUSED,
 					     jbyteArray vmData,
 					     jobject instance)
 {
@@ -555,7 +544,8 @@ Java_java_lang_reflect_Field_nativeGetInt
  * Signature: ([BLjava/lang/Object;)I
  */
 JNIEXPORT static jint JNICALL
-Java_java_lang_reflect_Field_nativeGetInt (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetInt (JNIEnv *_env,
+					   jobject this SVM_UNUSED,
 					   jbyteArray vmData,
 					   jobject instance)
 {
@@ -636,7 +626,8 @@ Java_java_lang_reflect_Field_nativeGetLong
  * Signature: ([BLjava/lang/Object;)J
  */
 JNIEXPORT static jlong JNICALL
-Java_java_lang_reflect_Field_nativeGetLong (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetLong (JNIEnv *_env,
+					    jobject this SVM_UNUSED,
 					    jbyteArray vmData,
 					    jobject instance)
 {
@@ -724,7 +715,8 @@ Java_java_lang_reflect_Field_nativeGetFloat
  * Signature: ([BLjava/lang/Object;)F
  */
 JNIEXPORT static jfloat JNICALL
-Java_java_lang_reflect_Field_nativeGetFloat (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetFloat (JNIEnv *_env,
+					     jobject this SVM_UNUSED,
 					     jbyteArray vmData,
 					     jobject instance)
 {
@@ -822,7 +814,8 @@ Java_java_lang_reflect_Field_nativeGetDouble
  * Signature: ([BLjava/lang/Object;)D
  */
 JNIEXPORT static jdouble JNICALL
-Java_java_lang_reflect_Field_nativeGetDouble (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeGetDouble (JNIEnv *_env,
+					      jobject this SVM_UNUSED,
 					      jbyteArray vmData,
 					      jobject instance)
 {
@@ -924,7 +917,8 @@ Java_java_lang_reflect_Field_nativeSetReference
  * Signature: ([BLjava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;
  */
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetReference (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetReference (JNIEnv *_env,
+						 jobject this SVM_UNUSED,
 						 jbyteArray vmData,
 						 jobject instance,
 						 jobject value)
@@ -970,7 +964,8 @@ Java_java_lang_reflect_Field_nativeSetBoolean
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetBoolean (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetBoolean (JNIEnv *_env,
+					       jobject this SVM_UNUSED,
 					       jbyteArray vmData,
 					       jobject instance,
 					       jboolean value)
@@ -1014,7 +1009,8 @@ Java_java_lang_reflect_Field_nativeSetByte
  * Signature: ([BLjava/lang/Object;B)V
  */
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetByte (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetByte (JNIEnv *_env,
+					    jobject this SVM_UNUSED,
 					    jbyteArray vmData,
 					    jobject instance, jbyte value)
 {
@@ -1102,7 +1098,8 @@ Java_java_lang_reflect_Field_nativeSetChar
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetChar (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetChar (JNIEnv *_env,
+					    jobject this SVM_UNUSED,
 					    jbyteArray vmData,
 					    jobject instance, jchar value)
 {
@@ -1183,7 +1180,8 @@ Java_java_lang_reflect_Field_nativeSetShort
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetShort (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetShort (JNIEnv *_env,
+					     jobject this SVM_UNUSED,
 					     jbyteArray vmData,
 					     jobject instance, jshort value)
 {
@@ -1267,7 +1265,8 @@ Java_java_lang_reflect_Field_nativeSetInt
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetInt (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetInt (JNIEnv *_env,
+					   jobject this SVM_UNUSED,
 					   jbyteArray vmData,
 					   jobject instance, jint value)
 {
@@ -1344,7 +1343,8 @@ Java_java_lang_reflect_Field_nativeSetLong
  */
 
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetLong (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetLong (JNIEnv *_env,
+					    jobject this SVM_UNUSED,
 					    jbyteArray vmData,
 					    jobject instance, jlong value)
 {
@@ -1417,7 +1417,8 @@ Java_java_lang_reflect_Field_nativeSetFloat
 
 
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetFloat (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetFloat (JNIEnv *_env,
+					     jobject this SVM_UNUSED,
 					     jbyteArray vmData,
 					     jobject instance, jfloat value)
 {
@@ -1495,7 +1496,8 @@ Java_java_lang_reflect_Field_nativeSetDouble
  * Signature: ([BLjava/lang/Object;D)V
  */
 JNIEXPORT static void JNICALL
-Java_java_lang_reflect_Field_nativeSetDouble (JNIEnv *_env, jobject this,
+Java_java_lang_reflect_Field_nativeSetDouble (JNIEnv *_env,
+					      jobject this SVM_UNUSED,
 					      jbyteArray vmData,
 					      jobject instance, jdouble value)
 {
