@@ -12,7 +12,7 @@ _svmf_encoding_length
 ----------------------------------------------------------------------
 */
 
-static size_t
+svm_static size_t
 _svmf_encoding_length (const char *name)
 {
   const char *p = name;
@@ -66,7 +66,7 @@ _svmf_hex_digit
 ----------------------------------------------------------------------
 */
 
-inline static char
+inline svm_static char
 _svmf_hex_digit (jint h)
 {
   assert ((h & 0x0f) == h);
@@ -85,7 +85,7 @@ _svmf_encode_name
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_encode_name (const char *name, char **dest)
 {
   const char *p = name;
@@ -175,7 +175,7 @@ _svmf_prepare_native_method_short_name
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_native_method_short_name (_svmt_JNIEnv *env,
 					_svmt_method_info *method)
 {
@@ -208,7 +208,7 @@ _svmf_prepare_native_method_long_name
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_native_method_long_name (_svmt_JNIEnv *env,
 				       _svmt_method_info *method)
 {
@@ -244,7 +244,7 @@ _svmf_prepare_native_method_args_count
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_prepare_native_method_args_count (_svmt_method_info *method)
 {
   char *p = DREF (method->descriptor, value) + 1;
@@ -308,7 +308,7 @@ _svmf_prepare_native_method_args
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_prepare_native_method_args (_svmt_method_info *method)
 {
   size_t i;
@@ -528,7 +528,7 @@ _svmf_prepare_native_ffi_args
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_native_ffi_args (_svmt_JNIEnv *env, _svmt_method_info *method)
 {
   _svmt_class_loader_info *class_loader =
@@ -577,7 +577,7 @@ _svmf_bind_native_method
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_bind_native_method (_svmt_JNIEnv *env, _svmt_method_info *method)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -585,10 +585,8 @@ _svmf_bind_native_method (_svmt_JNIEnv *env, _svmt_method_info *method)
     method->class_info->class_loader_info->native_library_list;
   jboolean monitor_acquired = JNI_FALSE;
 
-  if (_svmf_enter_object_monitor
-      (env,
-       *(vm->class_loading.boot_loader.classes.virtualmachine->
-	 class_instance)) != JNI_OK)
+  if (_svmf_enter_object_monitor (env, *(method->class_info->class_instance))
+      != JNI_OK)
     {
       goto error;
     }
@@ -601,7 +599,7 @@ _svmf_bind_native_method (_svmt_JNIEnv *env, _svmt_method_info *method)
       if (native_library->handle != NULL)
 	{
 	  /* try short name */
-	  method->data.native_method->code =
+	  method->data.native_method->code = SVM_CAST_EXTENSION
 	    (void (*)(void)) lt_dlsym (native_library->handle,
 				       method->data.native_method->
 				       short_name);
@@ -612,7 +610,7 @@ _svmf_bind_native_method (_svmt_JNIEnv *env, _svmt_method_info *method)
 	    }
 
 	  /* try long name */
-	  method->data.native_method->code =
+	  method->data.native_method->code = SVM_CAST_EXTENSION
 	    (void (*)(void)) lt_dlsym (native_library->handle,
 				       method->data.native_method->long_name);
 
@@ -667,21 +665,32 @@ _svmf_bind_native_method (_svmt_JNIEnv *env, _svmt_method_info *method)
     {
       /* method not found */
 
-      /***** START TEMPORARY! **********/
-      _svmf_printf (env, stderr, "*** Couldn't bind native method %s ***\n",
-		    method->data.native_method->short_name);
-      _svmf_printf (env, stderr, "*** or %s ***\n",
-		    method->data.native_method->long_name);
-      /***** END TEMPORARY! **********/
+      jstring function_name;
 
-      _svmf_error_UnsatisfiedLinkError (env);
+      if (_svmm_new_native_local (env, function_name) != JNI_OK)
+	{
+	  goto error;
+	}
+
+      if (_svmf_get_string
+	  (env, method->data.native_method->long_name,
+	   function_name) != JNI_OK)
+	{
+	  _svmm_free_native_local (env, function_name);
+	  goto error;
+	}
+
+      _svmf_error_UnsatisfiedLinkError_msg (env, function_name);
+
+      _svmm_free_native_local (env, function_name);
 
       {
 	/* pop frame */
 	_svmt_stack_frame *frame = env->stack.current_frame;
 
 	env->stack.current_frame =
-	  (_svmt_stack_frame *) (((char *) frame) - frame->previous_offset);
+	  (_svmt_stack_frame *) (void *) (((char *) frame) -
+					  frame->previous_offset);
       }
 
       goto error;
@@ -703,9 +712,7 @@ _svmf_bind_native_method (_svmt_JNIEnv *env, _svmt_method_info *method)
       monitor_acquired = JNI_FALSE;
 
       if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+	  (env, *(method->class_info->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -720,9 +727,7 @@ error:
       monitor_acquired = JNI_FALSE;
 
       if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+	  (env, *(method->class_info->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -737,15 +742,19 @@ _svmf_invoke_native_static
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_invoke_native_static (_svmt_JNIEnv *env)
 {
+#ifndef NDEBUG
   _svmt_JavaVM *vm = env->vm;
+#endif
+
   _svmt_stack_frame *frame = env->stack.current_frame;
   _svmt_method_info *method = frame->method;
   _svmt_method_frame_info *frame_info = method->frame_info;
   _svmt_stack_value *params =
-    (_svmt_stack_value *) (((char *) frame) - frame_info->start_offset);
+    (_svmt_stack_value *) (void *) (((char *) frame) -
+				    frame_info->start_offset);
   void **ptrs =
     (void **) &params[method->data.native_method->java_args_and_ret_count];
   jint param = 0;
@@ -754,8 +763,9 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
   void *nullvar = NULL;
 
   jint i;
-  _svmt_stack_native_reference *lrefs = (_svmt_stack_native_reference *)
-    (((char *) frame) + frame_info->end_offset);
+  _svmt_stack_native_reference *lrefs =
+    (_svmt_stack_native_reference *) (void *) (((char *) frame) +
+					       frame_info->end_offset);
   jint ref = 0;
 
   {
@@ -768,7 +778,9 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
     lrefs[-1].jint = lrefs_count;
     lrefs[-2].size_t = lrefs_size;
 
-    lrefs = (_svmt_stack_native_reference *) (((char *) lrefs) - lrefs_size);
+    lrefs =
+      (_svmt_stack_native_reference *) (void *) (((char *) lrefs) -
+						 lrefs_size);
 
     for (i = 0; i < lrefs_count; i++)
       {
@@ -888,7 +900,7 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
 
     if (env->vm->verbose_methods)
       {
-	_svmf_printf (env, stdout,
+	_svmf_printf (env, stderr,
 		      "[verbose methods: exiting  method %s.%s%s]\n",
 		      method->class_info->name, DREF (method->name, value),
 		      DREF (method->descriptor, value));
@@ -896,7 +908,7 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
 	{
 	  _svmt_method_info *caller_method;
 	  _svmt_stack_frame *caller_frame = env->stack.current_frame;
-	  caller_frame = (_svmt_stack_frame *)
+	  caller_frame = (_svmt_stack_frame *) (void *)
 	    (((char *) caller_frame) - caller_frame->previous_offset);
 	  caller_method = caller_frame->method;
 
@@ -904,7 +916,7 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
 		 (caller_method->access_flags, SVM_ACC_INTERNAL)
 		 && caller_method != &vm->stack_bottom_method)
 	    {
-	      caller_frame = (_svmt_stack_frame *)
+	      caller_frame = (_svmt_stack_frame *) (void *)
 		(((char *) caller_frame) - caller_frame->previous_offset);
 	      caller_method = caller_frame->method;
 	    }
@@ -912,7 +924,7 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
 	  if (!_svmf_is_set_flag
 	      (caller_method->access_flags, SVM_ACC_INTERNAL))
 	    {
-	      _svmf_printf (env, stdout,
+	      _svmf_printf (env, stderr,
 			    "[                   returning to %s.%s%s]\n",
 			    caller_method->class_info->name,
 			    DREF (caller_method->name, value),
@@ -928,11 +940,13 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
       {
 	/* retrieve and store return value */
 	params =
-	  (_svmt_stack_value *) (((char *) frame) - frame_info->start_offset);
+	  (_svmt_stack_value *) (void *) (((char *) frame) -
+					  frame_info->start_offset);
 
 	/* point "frame" at caller frame */
 	frame =
-	  (_svmt_stack_frame *) (((char *) frame) - frame->previous_offset);
+	  (_svmt_stack_frame *) (void *) (((char *) frame) -
+					  frame->previous_offset);
 
 	switch (method->data.native_method->ret_type)
 	  {
@@ -1117,12 +1131,13 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
 	      /* Of course, this test is only useful for a 128bit platform ;-) */
 	      if (sizeof (jlong) < sizeof (_svmt_word))
 		{
-		  *((jlong *) &params[0]) =
-		    (jlong) *((_svmt_word *) &result[0]);
+		  *((jlong *) (void *) &params[0]) =
+		    (jlong) *((_svmt_word *) (void *) &result[0]);
 		}
 	      else
 		{
-		  *((jlong *) &params[0]) = *((jlong *) &result[0]);
+		  *((jlong *) (void *) &params[0]) =
+		    *((jlong *) (void *) &result[0]);
 		}
 
 	      frame->stack_size += 2;
@@ -1140,12 +1155,13 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
 	      /* Of course, this test is only useful for a 128bit platform ;-) */
 	      if (sizeof (jdouble) < sizeof (_svmt_float_word))
 		{
-		  *((jdouble *) &params[0]) =
-		    (jdouble) *((_svmt_float_word *) &result[0]);
+		  *((jdouble *) (void *) &params[0]) =
+		    (jdouble) *((_svmt_float_word *) (void *) &result[0]);
 		}
 	      else
 		{
-		  *((jdouble *) &params[0]) = *((jdouble *) &result[0]);
+		  *((jdouble *) (void *) &params[0]) =
+		    *((jdouble *) (void *) &result[0]);
 		}
 
 	      frame->stack_size += 2;
@@ -1160,16 +1176,16 @@ end:
   frame = env->stack.current_frame;
 
   /* free native local frames */
-  lrefs = (_svmt_stack_native_reference *)
+  lrefs = (_svmt_stack_native_reference *) (void *)
     (((char *) frame) + frame->end_offset);
 
-  while (lrefs > ((_svmt_stack_native_reference *)
+  while (lrefs > ((_svmt_stack_native_reference *) (void *)
 		  (((char *) frame) + _svmv_stack_offset)))
     {
       jint lrefs_count = lrefs[-1].jint;
       size_t lrefs_size = lrefs[-2].size_t;
 
-      lrefs = (_svmt_stack_native_reference *)
+      lrefs = (_svmt_stack_native_reference *) (void *)
 	(((char *) lrefs) - lrefs_size);
 
       for (i = 0; i < lrefs_count; i++)
@@ -1183,7 +1199,8 @@ end:
 
   /* pop frame */
   env->stack.current_frame =
-    (_svmt_stack_frame *) (((char *) frame) - frame->previous_offset);
+    (_svmt_stack_frame *) (void *) (((char *) frame) -
+				    frame->previous_offset);
 
   /* is method synchronized? */
   if (method->synchronized)
@@ -1204,15 +1221,19 @@ _svmf_invoke_native_nonstatic
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 {
+#ifndef NDEBUG
   _svmt_JavaVM *vm = env->vm;
+
+#endif
   _svmt_stack_frame *frame = env->stack.current_frame;
   _svmt_method_info *method = frame->method;
   _svmt_method_frame_info *frame_info = method->frame_info;
   _svmt_stack_value *params =
-    (_svmt_stack_value *) (((char *) frame) - frame_info->start_offset);
+    (_svmt_stack_value *) (void *) (((char *) frame) -
+				    frame_info->start_offset);
   void **ptrs =
     (void **) &params[method->data.native_method->java_args_and_ret_count];
   jint param = 0;
@@ -1221,8 +1242,9 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
   void *nullvar = NULL;
 
   jint i;
-  _svmt_stack_native_reference *lrefs = (_svmt_stack_native_reference *)
-    (((char *) frame) + frame_info->end_offset);
+  _svmt_stack_native_reference *lrefs =
+    (_svmt_stack_native_reference *) (void *) (((char *) frame) +
+					       frame_info->end_offset);
   jint ref = 0;
 
   {
@@ -1235,7 +1257,9 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
     lrefs[-1].jint = lrefs_count;
     lrefs[-2].size_t = lrefs_size;
 
-    lrefs = (_svmt_stack_native_reference *) (((char *) lrefs) - lrefs_size);
+    lrefs =
+      (_svmt_stack_native_reference *) (void *) (((char *) lrefs) -
+						 lrefs_size);
 
     for (i = 0; i < lrefs_count; i++)
       {
@@ -1357,7 +1381,7 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 
     if (env->vm->verbose_methods)
       {
-	_svmf_printf (env, stdout,
+	_svmf_printf (env, stderr,
 		      "[verbose methods: exiting  method %s.%s%s]\n",
 		      method->class_info->name, DREF (method->name, value),
 		      DREF (method->descriptor, value));
@@ -1365,7 +1389,7 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 	{
 	  _svmt_method_info *caller_method;
 	  _svmt_stack_frame *caller_frame = env->stack.current_frame;
-	  caller_frame = (_svmt_stack_frame *)
+	  caller_frame = (_svmt_stack_frame *) (void *)
 	    (((char *) caller_frame) - caller_frame->previous_offset);
 	  caller_method = caller_frame->method;
 
@@ -1373,7 +1397,7 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 		 (caller_method->access_flags, SVM_ACC_INTERNAL)
 		 && caller_method != &vm->stack_bottom_method)
 	    {
-	      caller_frame = (_svmt_stack_frame *)
+	      caller_frame = (_svmt_stack_frame *) (void *)
 		(((char *) caller_frame) - caller_frame->previous_offset);
 	      caller_method = caller_frame->method;
 	    }
@@ -1381,7 +1405,7 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 	  if (!_svmf_is_set_flag
 	      (caller_method->access_flags, SVM_ACC_INTERNAL))
 	    {
-	      _svmf_printf (env, stdout,
+	      _svmf_printf (env, stderr,
 			    "[                   returning to %s.%s%s]\n",
 			    caller_method->class_info->name,
 			    DREF (caller_method->name, value),
@@ -1397,11 +1421,13 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
       {
 	/* retrieve and store return value */
 	params =
-	  (_svmt_stack_value *) (((char *) frame) - frame_info->start_offset);
+	  (_svmt_stack_value *) (void *) (((char *) frame) -
+					  frame_info->start_offset);
 
 	/* point "frame" at caller frame */
 	frame =
-	  (_svmt_stack_frame *) (((char *) frame) - frame->previous_offset);
+	  (_svmt_stack_frame *) (void *) (((char *) frame) -
+					  frame->previous_offset);
 
 	switch (method->data.native_method->ret_type)
 	  {
@@ -1586,12 +1612,13 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 	      /* Of course, this test is only useful for a 128bit platform ;-) */
 	      if (sizeof (jlong) < sizeof (_svmt_word))
 		{
-		  *((jlong *) &params[0]) =
-		    (jlong) *((_svmt_word *) &result[0]);
+		  *((jlong *) (void *) &params[0]) =
+		    (jlong) *((_svmt_word *) (void *) &result[0]);
 		}
 	      else
 		{
-		  *((jlong *) &params[0]) = *((jlong *) &result[0]);
+		  *((jlong *) (void *) &params[0]) =
+		    *((jlong *) (void *) &result[0]);
 		}
 
 	      frame->stack_size += 2;
@@ -1609,12 +1636,13 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 	      /* Of course, this test is only useful for a 128bit platform ;-) */
 	      if (sizeof (jdouble) < sizeof (_svmt_float_word))
 		{
-		  *((jdouble *) &params[0]) =
-		    (jdouble) *((_svmt_float_word *) &result[0]);
+		  *((jdouble *) (void *) &params[0]) =
+		    (jdouble) *((_svmt_float_word *) (void *) &result[0]);
 		}
 	      else
 		{
-		  *((jdouble *) &params[0]) = *((jdouble *) &result[0]);
+		  *((jdouble *) (void *) &params[0]) =
+		    *((jdouble *) (void *) &result[0]);
 		}
 
 	      frame->stack_size += 2;
@@ -1629,16 +1657,16 @@ end:
   frame = env->stack.current_frame;
 
   /* free native local frames */
-  lrefs = (_svmt_stack_native_reference *)
+  lrefs = (_svmt_stack_native_reference *) (void *)
     (((char *) frame) + frame->end_offset);
 
-  while (lrefs > ((_svmt_stack_native_reference *)
+  while (lrefs > ((_svmt_stack_native_reference *) (void *)
 		  (((char *) frame) + _svmv_stack_offset)))
     {
       jint lrefs_count = lrefs[-1].jint;
       size_t lrefs_size = lrefs[-2].size_t;
 
-      lrefs = (_svmt_stack_native_reference *)
+      lrefs = (_svmt_stack_native_reference *) (void *)
 	(((char *) lrefs) - lrefs_size);
 
       for (i = 0; i < lrefs_count; i++)
@@ -1652,7 +1680,8 @@ end:
 
   /* pop frame */
   env->stack.current_frame =
-    (_svmt_stack_frame *) (((char *) frame) - frame->previous_offset);
+    (_svmt_stack_frame *) (void *) (((char *) frame) -
+				    frame->previous_offset);
 
   /* is method synchronized? */
   if (method->synchronized)
