@@ -18,7 +18,7 @@ _svmh_new_object_instance
 #define _svmm_new_object_instance(env, class_info, instance) \
 _svmh_new_object_instance(env, class_info, &instance)
 
-static jint
+svm_static jint
 _svmh_new_object_instance (_svmt_JNIEnv *env, _svmt_class_info *class_info,
 			   _svmt_object_instance **pinstance);
 
