@@ -12,7 +12,7 @@ _svmf_prepare_interface_instanceof
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_interface_instanceof (_svmt_JNIEnv *env,
 				    _svmt_class_info *interface)
 {
@@ -68,7 +68,7 @@ _svmf_prepare_noninterface_instanceof
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_noninterface_instanceof (_svmt_JNIEnv *env,
 				       _svmt_class_info *class)
 {
@@ -192,7 +192,7 @@ _svmf_prepare_interface_fields
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_interface_fields (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
   jint i;
@@ -291,7 +291,7 @@ _svmf_save_instance_free_space
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_save_instance_free_space (_svmt_class_info *class, size_t size,
 				size_t offset)
 {
@@ -317,7 +317,7 @@ _svmf_get_instance_free_space
 ----------------------------------------------------------------------
 */
 
-static size_t
+svm_static size_t
 _svmf_get_instance_free_space (_svmt_class_info *class, size_t size)
 {
   jint i;
@@ -367,7 +367,7 @@ _svmf_prepare_noninterface_fields
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_noninterface_fields (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
   if (CAN_DREF (class->super_class))
@@ -655,7 +655,7 @@ _svmf_prepare_method_args_count
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_method_args_count (_svmt_JNIEnv *env, _svmt_method_info *method)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -884,7 +884,7 @@ _svmh_get_interface_method_id
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmh_get_interface_method_id (_svmt_JNIEnv *env, const char *name,
 			       const char *descriptor, jint *pmethod_id)
 {
@@ -934,7 +934,7 @@ _svmf_prepare_interface_methods
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_interface_methods (_svmt_JNIEnv *env,
 				 _svmt_class_info *interface)
 {
@@ -1075,7 +1075,7 @@ _svmf_prepare_noninterface_methods
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_noninterface_methods (_svmt_JNIEnv *env,
 				    _svmt_class_info *class)
 {
@@ -1114,7 +1114,7 @@ _svmf_prepare_noninterface_methods (_svmt_JNIEnv *env,
     {
       _svmt_method_info *method = &class->methods[i];
 
-      /* skip <init>,<clinit>, and static methods */
+      /* skip <init>,<clinit>, and svm_static methods */
       if (DREF (method->name, value)[0] == '<' ||
 	  _svmf_is_set_flag (method->access_flags, SVM_ACC_STATIC))
 	{
@@ -1308,7 +1308,7 @@ _svmf_prepare_interface
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_interface (_svmt_JNIEnv *env, _svmt_class_info *interface)
 {
   if (_svmf_prepare_interface_instanceof (env, interface) != JNI_OK)
@@ -1336,7 +1336,7 @@ _svmf_fill_interface_table_with_interface
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_fill_interface_table_with_interface (_svmt_class_info *interface,
 					   _svmt_method_info **imethods,
 					   _svmt_method_info **vmethods,
@@ -1399,7 +1399,7 @@ _svmf_fill_interface_table
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_fill_interface_table (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
   _svmt_u8 *visited;
@@ -1455,10 +1455,12 @@ _svmf_prepare_noninterface_vtable
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_noninterface_vtable (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
+#ifdef STATISTICS
   _svmt_JavaVM *vm = env->vm;
+#endif
   jboolean is_abstract =
     _svmf_is_set_flag (class->access_flags, SVM_ACC_ABSTRACT);
 
@@ -1535,7 +1537,7 @@ _svmf_prepare_noninterface_vtable (_svmt_JNIEnv *env, _svmt_class_info *class)
 
     for (i = 0; i < size; i++)
       {
-	/* skip <init>, <clinit>, and static methods */
+	/* skip <init>, <clinit>, and svm_static methods */
 	if (DREF (class->methods[i].name, value)[0] == '<' ||
 	    _svmf_is_set_flag (class->methods[i].access_flags,
 			       SVM_ACC_STATIC))
@@ -1602,7 +1604,7 @@ _svmf_prepare_noninterface
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_noninterface (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
   if (_svmf_prepare_noninterface_instanceof (env, class) != JNI_OK)
@@ -1636,10 +1638,12 @@ _svmf_prepare_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_class (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
+#ifdef STATISTICS
   _svmt_JavaVM *vm = env->vm;
+#endif
 
   assert (_svmf_is_set_flag (class->state, SVM_TYPE_STATE_VERIFIED));
 
@@ -1740,7 +1744,7 @@ _svmf_prepare_array_vtable
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_array_vtable (_svmt_JNIEnv *env, _svmt_array_info *array)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -1774,10 +1778,12 @@ _svmf_prepare_array
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_prepare_array (_svmt_JNIEnv *env, _svmt_array_info *array)
 {
+#ifdef STATISTICS
   _svmt_JavaVM *vm = env->vm;
+#endif
 
   if (_svmf_is_set_flag (array->state, SVM_TYPE_STATE_PREPARED))
     {
