@@ -33,8 +33,8 @@ _svmf_iflush
    inline-threading engine to work on such relatively common
    processors. */
 
-static inline void
-_svmf_iflush (_svmt_word *pword)
+svm_static inline void
+_svmf_iflush (_svmt_word *pword SVM_UNUSED)
 {
 /* *INDENT-OFF* */
 #if defined (__alpha__)
@@ -87,7 +87,7 @@ _svmf_iflush (_svmt_word *pword)
 			         
 #elif defined (_SABLEVM_INLINED_THREADED_INTERPRETER)
 #if defined (HAS_SYSTEM_CLEAR_CACHE)
-  __clear_cache(pword, pword + 1);
+  __clear_cache((char *) pword, (char *) (pword + 1));
 #else
 #error "SableVM is not prepared to run the inlined threaded interpreter on this kind of a system or __clear_cache is not available."
 #endif
@@ -117,7 +117,7 @@ _svmh_compare_and_swap (&word, old_value, new_value)
      }
 */
 
-static inline jboolean
+svm_static inline jboolean
 _svmh_compare_and_swap (volatile _svmt_word *pword, _svmt_word old_value,
 			_svmt_word new_value)
 {
@@ -463,7 +463,7 @@ reference fields is embedded in the object header.
 
 */
 
-static void
+svm_static void
 _svmf_prepare_class_lockword (_svmt_class_info *class)
 {
   _svmt_word lockword = 0;
@@ -572,7 +572,7 @@ _svmf_prepare_array_lockword
 
 */
 
-static void
+svm_static void
 _svmf_prepare_array_lockword (_svmt_array_info *array)
 {
   _svmt_word lockword = 0;
