@@ -20,7 +20,7 @@ _svmf_thread_init
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_thread_init (void)
 {
   /* create key for thread specific data */
@@ -70,7 +70,7 @@ _svmf_stack_init_defaults
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_stack_init_defaults (_svmt_JavaVM *vm)
 {
   vm->stack_min_size = SVM_STACK_DEFAULT_MIN_SIZE;
@@ -84,7 +84,7 @@ _svmf_stack_init
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_stack_init (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -132,15 +132,20 @@ _svmf_stack_init (_svmt_JNIEnv *env)
   env->stack.current_frame->stack_size = 0;
 
   {
-    _svmt_stack_native_reference *lrefs = (_svmt_stack_native_reference *)
-      (((char *) env->stack.current_frame) +
-       env->stack.current_frame->end_offset);
+    _svmt_stack_native_reference *lrefs =
+      (_svmt_stack_native_reference *) (void
+					*) (((char *) env->stack.
+					     current_frame) +
+					    env->stack.current_frame->
+					    end_offset);
     jint i;
 
     lrefs[-1].jint = lrefs_count;
     lrefs[-2].size_t = lrefs_size;
 
-    lrefs = (_svmt_stack_native_reference *) (((char *) lrefs) - lrefs_size);
+    lrefs =
+      (_svmt_stack_native_reference *) (void *) (((char *) lrefs) -
+						 lrefs_size);
 
     for (i = 0; i < lrefs_count; i++)
       {
@@ -160,7 +165,7 @@ _svmf_ensure_stack_capacity
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_ensure_stack_capacity (_svmt_JNIEnv *env, size_t frame_size)
 {
   void *current_frame = env->stack.current_frame;
@@ -249,8 +254,8 @@ _svmf_ensure_stack_capacity (_svmt_JNIEnv *env, size_t frame_size)
 	  env->stack.start = new_stack;
 	  env->stack.end = ((char *) new_stack) + new_size;
 	  env->stack.current_frame =
-	    (_svmt_stack_frame *) (((char *) new_stack) +
-				   current_frame_offset);
+	    (_svmt_stack_frame *) (void *) (((char *) new_stack) +
+					    current_frame_offset);
 	}
     }
 
@@ -266,7 +271,7 @@ _svmf_halt_if_requested
 /* IMPORTANT: The calling thread should hold the lock on
    vm->global_mutex when calling this function. */
 
-static void
+svm_static void
 _svmf_halt_if_requested (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -295,65 +300,27 @@ _svmf_halt_if_requested (_svmt_JNIEnv *env)
   assert (env->thread_status == SVM_THREAD_STATUS_RUNNING_JAVA);
 }
 
-#ifdef _SABLEVM_INLINED_THREADED_INTERPRETER
-
-#define _svmm_periodic_check_safe_jump(env) _svmf_periodic_check_safe_jump(env)
-
-static void (*_svmf_periodic_check_halt_requested_ptr) (_svmt_JNIEnv *);
-
 /*
 ----------------------------------------------------------------------
 _svmf_periodic_check_halt_requested
 ----------------------------------------------------------------------
 */
-static void
+
+#ifdef COMMENT
+
+svm_static void
 _svmf_periodic_check_halt_requested (_svmt_JNIEnv *env)
 {
-#ifdef _SABLEVM_INLINABILITY_TESTING
-  if (env->thread_halt_requested_is_faked == JNI_TRUE)
-    {
-      env->thread_halt_requested_is_faked = JNI_FALSE;
-      env->thread_status = SVM_THREAD_STATUS_RUNNING_JAVA;
-      return;
-    }
-#endif
+  _svmt_JavaVM *vm = env->vm;
 
-  {
-    _svmt_JavaVM *vm = env->vm;
+  _svmm_mutex_lock (vm->global_mutex);
 
-    _svmm_mutex_lock (vm->global_mutex);
-
-    _svmf_halt_if_requested (env);
-
-    _svmm_mutex_unlock ();
-  }
+  _svmf_halt_if_requested (env);
 
+  _svmm_mutex_unlock ();
 }
 
-/*
-----------------------------------------------------------------------
-_svmf_periodic_check_safe_jump
-----------------------------------------------------------------------
-*/
-
-inline static void
-_svmf_periodic_check_safe_jump (_svmt_JNIEnv *env)
-{
-#ifndef NDEBUG
-  jint status = env->thread_status;
-  assert (status == SVM_THREAD_STATUS_RUNNING_JAVA ||
-	  status == SVM_THREAD_STATUS_HALT_REQUESTED);
-#endif
-
-  /* is stopping the world requested by another thread? */
-  if (env->thread_status == SVM_THREAD_STATUS_HALT_REQUESTED)
-    _svmf_periodic_check_halt_requested_ptr (env);
-}
-#else /* _SABLEVM_INLINED_THREADED_INTERPRETER */
-
-#define _svmm_periodic_check_safe_jump(env) _svmf_periodic_check(env)
-
-#endif /* _SABLEVM_INLINED_THREADED_INTERPRETER */
+#endif /* COMMENT */
 
 /*
 ----------------------------------------------------------------------
@@ -361,7 +328,7 @@ _svmf_periodic_check
 ----------------------------------------------------------------------
 */
 
-inline static void
+inline svm_static void
 _svmf_periodic_check (_svmt_JNIEnv *env)
 {
 #ifndef NDEBUG
@@ -394,7 +361,7 @@ _svmf_stop_the_world
    stopped, the calling thread should release the lock until just
    prior the call to resume_the_world. */
 
-static void
+svm_static void
 _svmf_stop_the_world (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -483,7 +450,7 @@ _svmf_resume_the_world
    function should not be invoked unless the world has been already
    stopped by the calling thread. */
 
-static void
+svm_static void
 _svmf_resume_the_world (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -545,7 +512,7 @@ _svmf_inflate_lock_no_exception
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_inflate_lock_no_exception (_svmt_JNIEnv *env,
 				 _svmt_object_instance *instance)
 {
@@ -623,7 +590,7 @@ _svmf_stopping_java
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_stopping_java (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -657,7 +624,7 @@ _svmf_resuming_java
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_resuming_java (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -696,12 +663,15 @@ _svmf_enter_object_monitor
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_enter_object_monitor (_svmt_JNIEnv *env,
 			    _svmt_object_instance *instance)
 {
   _svmt_JavaVM *vm = env->vm;
   _svmt_word old_lockword;
+#ifndef NDEBUG
+  const char *type_name = instance->vtable->type->name;
+#endif
 
   assert (instance != NULL);
 
@@ -834,6 +804,21 @@ retry:
 	  /* while thread is still on the owner's wait list, then
 	     wait */
 
+#ifndef NDEBUG
+	  if (vm->verbose_synchronization)
+	    {
+	      /* dump a stack trace on contention */
+
+	      _svmf_printf (env, stderr, "*** CONTENTION ***\n");
+	      _svmf_printf (env, stderr,
+			    "Thread (%d,%d) already owns the thin monitor of type %s seeked by thread (%d,%d)\n",
+			    owner->thread.id, owner->thread.pthread,
+			    type_name, env->thread.id, env->thread.pthread);
+
+	      _svmf_dump_stack_trace (env);
+	    }
+#endif
+
 	  do
 	    {
 	      _svmt_JNIEnv *current = owner->contention.owner.wait_list;
@@ -856,6 +841,18 @@ retry:
 	    }
 	  while (JNI_TRUE);
 
+#ifndef NDEBUG
+	  if (vm->verbose_synchronization)
+	    {
+	      /* tell about resolved contention */
+
+	      _svmf_printf (env, stderr, "*** RESOLVED CONTENTION ***\n");
+	      _svmf_printf (env, stderr,
+			    "Thread (%d,%d) has finally got hold of the monitor.\n",
+			    env->thread.id, env->thread.pthread);
+	    }
+#endif
+
 	  _svmm_mutex_unlock ();
 
 	  _svmf_resuming_java (env);
@@ -893,16 +890,72 @@ retry:
 
       _svmm_mutex_lock (fat_lock->mutex);
 
-      /* wait until no other thread owns the lock */
-      while (fat_lock->recursive_count != 0 && fat_lock->owner != env)
-	{
-	  _svmm_cond_wait (fat_lock->cond, fat_lock->mutex);
-	}
+#ifndef NDEBUG
+      /* dump a stack trace on contention */
+
+      {
+	jboolean contention_happened = JNI_FALSE;
+
+	if (vm->verbose_synchronization)
+	  {
+	    if (fat_lock->recursive_count != 0 && fat_lock->owner != env)
+	      {
+		contention_happened = JNI_TRUE;
+
+		_svmf_printf (env, stderr, "*** CONTENTION ***\n");
+		_svmf_printf (env, stderr,
+			      "Thread (%d,%d) already owns the fat monitor %p of type %s seeked by thread (%d,%d)\n",
+			      fat_lock->owner->thread.id,
+			      fat_lock->owner->thread.pthread, fat_lock,
+			      type_name, env->thread.id, env->thread.pthread);
+
+		_svmf_dump_stack_trace (env);
+
+	      }
+	  }
+#endif
+
+	/* wait until no other thread owns the lock */
+	while (fat_lock->recursive_count != 0 && fat_lock->owner != env)
+	  {
+	    _svmm_cond_wait (fat_lock->cond, fat_lock->mutex);
+	  }
+
+#ifndef NDEBUG
+	if (vm->verbose_synchronization)
+	  {
+	    /* tell about resolved contention */
+
+	    if (contention_happened)
+	      {
+		_svmf_printf (env, stderr, "*** RESOLVED CONTENTION ***\n");
+		_svmf_printf (env, stderr,
+			      "Thread (%d,%d) has finally got hold of the monitor.\n",
+			      env->thread.id, env->thread.pthread);
+	      }
+	  }
+      }
+#endif
 
       if (fat_lock->recursive_count == 0)
 	{
 	  fat_lock->recursive_count = 1;
 	  fat_lock->owner = env;
+
+#ifndef NDEBUG
+	  if (vm->verbose_synchronization)
+	    {
+	      /* dump a stack trace on fat lock acquisition */
+
+	      _svmf_printf (env, stderr, "*** FAT LOCK ACQUIRED ***\n");
+	      _svmf_printf (env, stderr,
+			    "Thread (%d,%d) aquires monitor %p of type %s\n",
+			    env->thread.id, env->thread.pthread, fat_lock,
+			    type_name);
+
+	      _svmf_dump_stack_trace (env);
+	    }
+#endif
 	}
       else
 	{
@@ -943,7 +996,7 @@ _svmf_exit_object_monitor
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_exit_object_monitor (_svmt_JNIEnv *env, _svmt_object_instance *instance)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -1085,7 +1138,7 @@ _svmf_thread_start
 ----------------------------------------------------------------------
 */
 
-static void *
+svm_static void *
 _svmf_thread_start (void *_env)
 {
   _svmt_JNIEnv *env = (_svmt_JNIEnv *) _env;
