@@ -42,6 +42,9 @@ typedef struct _svmt_arg_preparation_struct _svmt_arg_preparation;
 typedef struct _svmt_array_info_struct _svmt_array_info;
 typedef struct _svmt_array_instance_struct _svmt_array_instance;
 typedef struct _svmt_attribute_info_struct _svmt_attribute_info;
+typedef struct _svmt_zip_file_struct _svmt_zip_file;
+typedef struct _svmt_zip_file_entry_struct _svmt_zip_file_entry;
+typedef struct _svmt_boot_class_path_entry_struct _svmt_boot_class_path_entry;
 typedef struct _svmt_bootstrap_classloader_struct _svmt_bootstrap_classloader;
 typedef struct _svmt_branch_preparation_struct _svmt_branch_preparation;
 typedef struct _svmt_bytecode_info_struct _svmt_bytecode_info;
@@ -105,6 +108,10 @@ typedef union _svmt_code_union _svmt_code;
 typedef union _svmt_stack_native_reference_union _svmt_stack_native_reference;
 typedef union _svmt_stack_value_union _svmt_stack_value;
 
+#ifdef _SABLEVM_INLINABILITY_TESTING
+typedef struct _svmt_inlinability_testing_struct _svmt_inlinability_testing;
+#endif
+
 /* *INDENT-ON* */
 
 /*
@@ -159,6 +166,25 @@ struct _svmt_stack_struct
   _svmt_stack_frame *current_frame;
 };
 
+/*
+----------------------------------------------------------------------
+_svmt_inlinability_testing
+----------------------------------------------------------------------
+*/
+
+#ifdef _SABLEVM_INLINABILITY_TESTING
+
+struct _svmt_inlinability_testing_struct
+{
+  jbyte bytecode_inlinability[SVM_INSTRUCTION_COUNT];
+  jbyte bytecode_recognizability[SVM_INSTRUCTION_COUNT];
+  jboolean ever_used;
+  jint last_signal;
+  volatile jint sigsegv_expected;
+};
+
+#endif
+
 /*
 ----------------------------------------------------------------------
 _svmt_JNIEnv
@@ -187,6 +213,9 @@ struct _svmt_JNIEnv_struct
    */
   jthrowable throwable;
 
+  /* number of time a "critical" JNI function was entered */
+  jint critical_section_count;
+
 #if defined(_SABLEVM_SIGNALS_FOR_EXCEPTIONS) || defined(_SABLEVM_INLINABILITY_TESTING)
 
   /* We maintain a pointer to the innermost exception handler.
@@ -199,11 +228,6 @@ struct _svmt_JNIEnv_struct
   sigjmp_buf *volatile signal_handler;
   volatile jint signal_code;
 
-#ifdef _SABLEVM_INLINABILITY_TESTING
-  volatile jint inlinability_testing_sigsegv_expected;
-  volatile jint thread_halt_requested_is_faked;
-#endif
-
 #ifndef NDEBUG
 
   volatile jint sigsegv_expected;
@@ -211,6 +235,12 @@ struct _svmt_JNIEnv_struct
 
 #endif				/* not NDEBUG */
 
+#ifdef _SABLEVM_INLINABILITY_TESTING
+
+  _svmt_inlinability_testing inlinability_testing;
+
+#endif
+
 #endif				/* _SABLEVM_SIGNALS_FOR_EXCEPTIONS || _SABLEVM_INLINABILITY_TESTING */
 
   /* status for "stop_the_world" operations */
@@ -261,22 +291,23 @@ struct _svmt_JNIEnv_struct
 _svmt_classes
 ----------------------------------------------------------------------
 */
-
 struct _svmt_classes_struct
 {
   _svmt_class_info *jiserializable;
   _svmt_class_info *jlclass;
-  _svmt_class_info *jlvmclass;
-  _svmt_class_info *gcrawdata;
   _svmt_class_info *jlcloneable;
   _svmt_class_info *jlobject;
   _svmt_class_info *jlrconstructor;
   _svmt_class_info *jlrfield;
+  _svmt_class_info *jlrinvocationtargetexception;
   _svmt_class_info *jlrmethod;
   _svmt_class_info *jlstacktraceelement;
+  _svmt_class_info *jlvmclass;
   _svmt_class_info *stringcreator;
   _svmt_class_info *virtualmachine;
 
+  _svmt_array_info *jlrfield_array;
+
   /* primitive arrays */
   _svmt_array_info *boolean_array;
   _svmt_array_info *byte_array;
@@ -304,16 +335,21 @@ struct _svmt_methods_struct
   _svmt_method_info *jlclass_initialize;
   _svmt_method_info *jlrconstructor_init;
   _svmt_method_info *jlrfield_init;
+  _svmt_method_info *jlrinvocationtargetexception_init;
   _svmt_method_info *jlrmethod_init;
   _svmt_method_info *jlstacktraceelement_init;
   _svmt_method_info *stringcreator_createinternedstring;
   _svmt_method_info *stringcreator_createstring;
+  _svmt_method_info *stringcreator_createstringfromchars;
+  _svmt_method_info *stringcreator_getchars;
+  _svmt_method_info *stringcreator_getlength;
   _svmt_method_info *stringcreator_getutfchars;
   _svmt_method_info *stringcreator_getutflength;
   _svmt_method_info *virtualmachine_createarray;
   _svmt_method_info *virtualmachine_createclass;
-  _svmt_method_info *virtualmachine_exceptiondescribe;
   _svmt_method_info *virtualmachine_createrootthread;
+  _svmt_method_info *virtualmachine_exceptiondescribe;
+  _svmt_method_info *virtualmachine_getsystemcl;
   _svmt_method_info *virtualmachine_runthread;
 
 #include "error_init_methods.h"
@@ -376,9 +412,52 @@ _svmt_class_file
 struct _svmt_class_file_struct
 {
   jint length;
+  jboolean free_bytes_after_use;
   _svmt_u8 *bytes;
 };
 
+/*
+----------------------------------------------------------------------
+_svmt_zip_file_struct
+----------------------------------------------------------------------
+*/
+
+struct _svmt_zip_file_struct
+{
+  int fd;
+  char *name;
+  _svmt_u32 file_len;
+  _svmt_u8 *content;		/* mmaped */
+  _svmt_u16 entries_no;
+  _svmt_zip_file_entry *entries;
+};
+
+/*
+----------------------------------------------------------------------
+_svmt_zip_file_entry_struct
+----------------------------------------------------------------------
+*/
+
+struct _svmt_zip_file_entry_struct
+{
+  char *name;
+  _svmt_u32 local_header_offset;
+};
+
+/*
+----------------------------------------------------------------------
+_svmt_boot_class_path_entry
+----------------------------------------------------------------------
+*/
+
+struct _svmt_boot_class_path_entry_struct
+{
+  jboolean is_valid;		/* non-existan dir/files, or non-zip files get this flag set */
+  jboolean is_file;		/* class_path should point to a directory or zip/jar file */
+  char *class_path;
+  _svmt_zip_file *zip_file;	/* only if not NULL for non-zip/jar class_path */
+};
+
 /*
 ----------------------------------------------------------------------
 _svmt_bootstrap_classloader
@@ -387,9 +466,13 @@ _svmt_bootstrap_classloader
 
 struct _svmt_bootstrap_classloader_struct
 {
-  char *boot_class_path;
+  char *boot_class_path, *boot_class_path_prepend, *boot_class_path_append;
   char *boot_library_path;
 
+  /* boot_class_path split in elements separated by ':' */
+  _svmt_boot_class_path_entry *boot_class_path_entries;
+  jint boot_class_path_entries_no;
+
   /* system properties received through the VM creation structure. */
   _svmt_system_properties system_properties;
 
@@ -400,7 +483,6 @@ struct _svmt_bootstrap_classloader_struct
 
   _svmt_type_node *initiated_type_tree;
   _svmt_type_node *partially_derived_type_tree;
-  _svmt_class_file current_class_file;
 
   _svmt_internal_method_node *internal_method_tree;
 
@@ -712,6 +794,21 @@ struct _svmt_native_globals_struct
   _svmt_native_ref *free_list;
 };
 
+/*
+----------------------------------------------------------------------
+_svmt_remember_set
+----------------------------------------------------------------------
+*/
+#if defined (_SABLEVM_GENCOPY_GC)
+struct _svmt_remember_set_struct
+{
+  struct _svmt_remember_set_struct *next;
+  void *ref_hash[SVM_HEAP_REMSET_BUCKET_NUM][SVM_HEAP_REMSET_BUCKET_SIZE];
+};
+
+typedef struct _svmt_remember_set_struct _svmt_remember_set;
+#endif /* _SABLEVM_GENCOPY_GC */
+
 /*
 ----------------------------------------------------------------------
 _svmt_heap
@@ -730,7 +827,7 @@ struct _svmt_heap_struct
   void *alloc;			/* current allocation position */
   jint hashcode_base;
 
-#elif defined (_SABLEVM_COPY_GC)
+#elif defined (_SABLEVM_COPY_GC) || defined (_SABLEVM_GENCOPY_GC)
 
   size_t min_size;
   size_t max_size;		/* 0 means no max */
@@ -749,6 +846,23 @@ struct _svmt_heap_struct
   long total_gc_secs;
   long total_gc_usecs;
 
+#if defined (_SABLEVM_GENCOPY_GC)
+
+  int global_gc;
+
+  size_t nursery_size;
+  void *nursery_start;
+  void *nursery_end;
+  void *nursery_alloc;
+
+  size_t large_object_size;
+
+  _svmt_remember_set remember_set;
+
+  jint hashcode_base_nursery;
+  size_t hashed_notmoved_nursery;	/* number of objects that will inflate on next Local GC */
+
+#endif				/* defined (_SABLEVM_GENCOPY_GC) */
 #endif				/* defined (_SABLEVM_NO_GC) */
 };
 
@@ -768,7 +882,8 @@ struct _svmt_instruction_info_struct
   size_t inlined_size;
 #endif
 #if !defined(NDEBUG) || defined (_SABLEVM_INLINED_THREADED_INTERPRETER)
-  char name[30];
+  /* See comment about static allocation in inlined_testing.c file. */
+  char name[_SABLEVM_BYTECODE_NAME_MAX_LENGTH];
 #endif
 };
 
@@ -916,10 +1031,18 @@ struct _svmt_JavaVM_struct
   jboolean verbose_gc;
   jboolean verbose_jni;
 
+#if !defined(NDEBUG)
+
+  jboolean verbose_exceptions;	/* dump stack trace for every thrown exception */
+  jboolean exit_stack_trace;	/* dump stack trace on java.lang.Runtime.exit() */
+
+#endif
+
 #if !defined(NDEBUG) || defined(_SABLEVM_VERBOSE_INSTRUCTIONS_INLINED)
 
   jboolean verbose_methods;
   jboolean verbose_instructions;
+  jboolean verbose_synchronization;
 
 #endif
 
@@ -1164,6 +1287,7 @@ struct _svmt_class_info_struct
   jint state;			/* SVM_TYPE_STATE_XXX mask */
   jthrowable preparation_error;
   jthrowable verification_error;
+  jobject protection_domain;
 
   jint minor_version;
   jint major_version;
@@ -1281,6 +1405,7 @@ struct _svmt_CONSTANT_String_info_struct
   jint tag;
   _svmt_CONSTANT_Utf8_info **string;
 
+  _svmt_class_info *class_info;
   jstring value;
 };
 
