@@ -12,7 +12,7 @@ _svmf_bytes_to_u16
 ----------------------------------------------------------------------
 */
 
-inline static _svmt_u16
+inline svm_static _svmt_u16
 _svmf_bytes_to_u16 (_svmt_u8 *bytes)
 {
   jint i;
@@ -32,7 +32,7 @@ _svmf_bytes_to_s32
 ----------------------------------------------------------------------
 */
 
-inline static _svmt_s32
+inline svm_static _svmt_s32
 _svmf_bytes_to_s32 (_svmt_u8 *bytes)
 {
   jint i;
@@ -52,7 +52,7 @@ _svmf_bytes_to_s16
 ----------------------------------------------------------------------
 */
 
-inline static _svmt_s16
+inline svm_static _svmt_s16
 _svmf_bytes_to_s16 (_svmt_u8 *bytes)
 {
   jint i;
@@ -192,13 +192,23 @@ _svmh_validate_min_max_increment
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmh_validate_min_max_increment (size_t *pmin, size_t *pmax, size_t *pincr)
 {
   *pmin = _svmf_aligned_size_t (*pmin);
   *pmax = _svmf_aligned_size_t (*pmax);
   *pincr = _svmf_aligned_size_t (*pincr);
 
+  /* Allow shortcut for specifying fixed size */
+  if (*pmin == *pmax)
+    {
+      *pincr = 0;
+    }
+  else if (*pincr == 0)
+    {
+      *pmax = *pmin;
+    }
+
   /* min must be positive */
   if (*pmin == 0)
     {
@@ -266,7 +276,7 @@ _svmf_set_boolean_array_element
 ----------------------------------------------------------------------
 */
 
-inline static void
+inline svm_static void
 _svmf_set_boolean_array_element (_svmt_array_instance *array, jint indx,
 				 jboolean value)
 {
@@ -297,7 +307,7 @@ _svmf_get_reference_array_element
 ----------------------------------------------------------------------
 */
 
-inline static _svmt_object_instance *
+inline svm_static _svmt_object_instance *
 _svmf_get_reference_array_element (_svmt_array_instance *array, jint indx)
 {
   _svmt_object_instance **elements;
@@ -329,14 +339,20 @@ _svmf_printf
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_printf (_svmt_JNIEnv *env, FILE *stream, const char *format, ...)
 {
   _svmt_JavaVM *vm = env->vm;
   va_list ap;
 
   va_start (ap, format);
+
   (*(vm->vfprintf)) (stream, format, ap);
+
+#if (!defined(NDEBUG)) || defined(_SABLEVM_INLINABILITY_TESTING)
+  fflush (NULL);
+#endif
+
   va_end (ap);
 }
 
@@ -346,7 +362,7 @@ _svmf_parse_size_t
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_parse_size_t (size_t *size, const char *value)
 {
   size_t result = 0;
@@ -424,7 +440,7 @@ inline static _svmt_object_instance *
 _svmf_get_REFERENCE_field (_svmt_object_instance *instance, size_t offset)
 {
   _svmt_object_instance *result =
-    *((_svmt_object_instance **) (((char *) instance) + offset));
+    *((_svmt_object_instance **) (void *) (((char *) instance) + offset));
 
 #if defined (MAGIC) && !defined (_SABLEVM_INLINED_THREADED_INTERPRETER)
   assert (result == NULL || strcmp (result->magic, "SableVM") == 0);
@@ -443,7 +459,8 @@ inline static void
 _svmf_put_REFERENCE_field (_svmt_object_instance *instance, size_t offset,
 			   _svmt_object_instance *value)
 {
-  *((_svmt_object_instance **) (((char *) instance) + offset)) = value;
+  *((_svmt_object_instance **) (void *) (((char *) instance) + offset)) =
+    value;
 }
 
 /*
@@ -482,7 +499,7 @@ _svmf_dump_stack_trace
 ----------------------------------------------------------------------
 */
 
-static void
+svm_static void
 _svmf_dump_stack_trace (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -492,6 +509,10 @@ _svmf_dump_stack_trace (_svmt_JNIEnv *env)
   jint lineNumber = -1;
   jboolean isNative = JNI_FALSE;
 
+  _svmf_printf (env, stderr, "--- stack trace dump (begin) ---\n");
+  _svmf_printf (env, stderr, "Thread ID: %d, Posix ID: %d\n", env->thread.id,
+		env->thread.pthread);
+
   while (method != &vm->stack_bottom_method)
     {
       _svmt_class_info *class;
@@ -499,7 +520,9 @@ _svmf_dump_stack_trace (_svmt_JNIEnv *env)
       /* skip internal frames */
       if (_svmf_is_set_flag (method->access_flags, SVM_ACC_INTERNAL))
 	{
-	  frame = (_svmt_stack_frame *)
+	  _svmf_printf (env, stderr, "(internal frame)\n");
+
+	  frame = (_svmt_stack_frame *) (void *)
 	    (((char *) frame) - frame->previous_offset);
 	  method = frame->method;
 	  continue;
@@ -538,10 +561,13 @@ _svmf_dump_stack_trace (_svmt_JNIEnv *env)
       lineNumber = -1;
       isNative = JNI_FALSE;
 
-      frame = (_svmt_stack_frame *)
+      frame = (_svmt_stack_frame *) (void *)
 	(((char *) frame) - frame->previous_offset);
       method = frame->method;
     }
+
+  _svmf_printf (env, stderr, "--- stack trace dump (end) ---\n");
+
 }
 
 
@@ -563,7 +589,7 @@ have_identical_package_name
 ----------------------------------------------------------------------
 */
 
-static jboolean
+svm_static jboolean
 have_identical_package_name (const char *name1, const char *name2)
 {
   size_t i, j;
