@@ -15,7 +15,7 @@ _svmm_identity_hashcode
 #define _svmm_identity_hashcode(env, obj, hashcode) \
 _svmh_identity_hashcode (env, obj, &hashcode)
 
-static jint
+svm_static jint
 _svmh_identity_hashcode (_svmt_JNIEnv *env, jobject obj, jint *phashcode)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -55,19 +55,35 @@ retry:
 	      goto retry;
 	    }
 
-#if defined (_SABLEVM_COPY_GC)
+#if defined (_SABLEVM_COPY_GC) || defined (_SABLEVM_GENCOPY_GC)
 
 	  /* we must make sure there will be enough space to store the hashcode at gc time */
 	  {
-	    jint status = JNI_OK;
+	    jint end_of_heap, status = JNI_OK;
 
 	    _svmm_mutex_lock (vm->global_mutex);
 
 	    /* remember that allocation is made in multiples of SVM_ALIGNMENT */
-	    if ((vm->heap.alloc != vm->heap.end) ||
-		((status =
-		  _svmf_copy_gc_no_exception (env, SVM_ALIGNMENT)) == JNI_OK))
+	    if (vm->heap.alloc == vm->heap.end)
+	      end_of_heap = JNI_TRUE;
+	    else
+	      end_of_heap = JNI_FALSE;
+
+	    if (end_of_heap == JNI_TRUE)
 	      {
+#if defined (_SABLEVM_COPY_GC)
+		status = _svmf_copy_gc_no_exception (env, SVM_ALIGNMENT);
+#elif defined (_SABLEVM_GENCOPY_GC)
+		status = _svmf_gencopy_gc_no_exception (env, SVM_ALIGNMENT);
+#endif
+	      }
+
+	    if ((end_of_heap == JNI_FALSE) || (status == JNI_OK))
+	      {
+#if defined (_SABLEVM_GENCOPY_GC)
+		if (_svmf_in_nursery (env, *obj))
+		  vm->heap.hashed_notmoved_nursery++;
+#endif
 		vm->heap.hashed_notmoved++;
 		vm->heap.end = ((char *) vm->heap.end) - SVM_ALIGNMENT;
 	      }
@@ -97,6 +113,13 @@ retry:
 
 	  *phashcode = vm->heap.hashcode_base + (size_t) instance;
 
+#elif defined (_SABLEVM_GENCOPY_GC)
+
+	  if (_svmf_in_nursery (env, instance))
+	    *phashcode = vm->heap.hashcode_base_nursery + (size_t) instance;
+	  else
+	    *phashcode = vm->heap.hashcode_base - (size_t) instance;
+
 #else
 #error todo
 #endif
@@ -109,6 +132,13 @@ retry:
 
 	  *phashcode = vm->heap.hashcode_base + (size_t) instance;
 
+#elif defined (_SABLEVM_GENCOPY_GC)
+
+	  if (_svmf_in_nursery (env, instance))
+	    *phashcode = vm->heap.hashcode_base_nursery + (size_t) instance;
+	  else
+	    *phashcode = vm->heap.hashcode_base - (size_t) instance;
+
 #else
 #error todo
 #endif
@@ -149,7 +179,7 @@ _svmf_is_assignable_from
 ----------------------------------------------------------------------
 */
 
-inline static jboolean
+inline svm_static jboolean
 _svmf_is_assignable_from (_svmt_JNIEnv *env, _svmt_type_info *from,
 			  _svmt_type_info *to)
 {
@@ -367,7 +397,7 @@ _svmf_set_reference_array_element_no_exception
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_set_reference_array_element_no_exception (_svmt_JNIEnv *env,
 						_svmt_array_instance *array,
 						jint indx,
@@ -417,6 +447,10 @@ _svmf_set_reference_array_element_no_exception (_svmt_JNIEnv *env,
 
   elements = (_svmt_object_instance **) array;
   elements[(-1) - indx] = value;
+#if defined (_SABLEVM_GC_WRITE_BARRIER)
+  _svmf_write_barrier
+    (env, (_svmt_object_instance **) &elements[(-1) - indx]);
+#endif /* _SABLEVM_GC_WRITE_BARRIER */
 
 #else
 
@@ -424,6 +458,9 @@ _svmf_set_reference_array_element_no_exception (_svmt_JNIEnv *env,
     (((char *) array) + _svmf_aligned_size_t (sizeof (_svmt_array_instance)));
 
   elements[indx] = value;
+#if defined (_SABLEVM_GC_WRITE_BARRIER)
+  _svmf_write_barrier (env, (_svmt_object_instance **) &elements[indx]);
+#endif /* _SABLEVM_GC_WRITE_BARRIER */
 
 #endif
 
@@ -436,7 +473,7 @@ _svmf_tree_compare_type
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_tree_compare_type (_svmt_type_node *value1, _svmt_type_node *value2)
 {
   assert (value1 != NULL);
@@ -451,7 +488,7 @@ _svmf_tree_compare_interface_method_signature
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_tree_compare_imethod_signature (_svmt_imethod_signature_node *value1,
 				      _svmt_imethod_signature_node *value2)
 {
@@ -471,7 +508,7 @@ _svmf_tree_compare_gc_map
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_tree_compare_gc_map (_svmt_gc_map_node *value1,
 			   _svmt_gc_map_node *value2)
 {
@@ -502,7 +539,7 @@ _svmf_tree_compare_sequence
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_tree_compare_sequence (_svmt_sequence_node *value1,
 			     _svmt_sequence_node *value2)
 {
@@ -532,7 +569,7 @@ _svmf_tree_compare_internal_method
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmf_tree_compare_internal_method (_svmt_internal_method_node *value1,
 				    _svmt_internal_method_node *value2)
 {
@@ -548,7 +585,7 @@ _svmh_global_copy_jobject
 ----------------------------------------------------------------------
 */
 
-inline static jint
+inline svm_static jint
 _svmh_global_copy_jobject (_svmt_JNIEnv *env, jobject src, jobject *dest)
 {
   assert (src != NULL);
@@ -571,7 +608,7 @@ _svmh_galloc_copy_str_no_exception
 #define _svmm_galloc_copy_str_no_exception(dst, src) \
 _svmh_galloc_copy_str_no_exception (&dst, src)
 
-static jint
+svm_static jint
 _svmh_galloc_copy_str_no_exception (char **dest, const char *src)
 {
   char *tmp;
@@ -600,7 +637,7 @@ _svmh_gfree_str_no_exception
 #define _svmm_gfree_str_no_exception(str) \
 _svmh_gfree_str_no_exception (&str)
 
-static void
+svm_static void
 _svmh_gfree_str_no_exception (char **str)
 {
   assert (*str != NULL);
@@ -618,7 +655,7 @@ _svmh_galloc_copy_str
 #define _svmm_galloc_copy_str(env, dst, src) \
 _svmh_galloc_copy_str (env, &dst, src)
 
-static jint
+svm_static jint
 _svmh_galloc_copy_str (_svmt_JNIEnv *env, char **dest, const char *src)
 {
   char *tmp;
@@ -648,7 +685,7 @@ _svmh_gfree_str
 #define _svmm_gfree_str(str) \
 _svmh_gfree_str (&str)
 
-static void
+svm_static void
 _svmh_gfree_str (char **str)
 {
   assert (*str != NULL);
@@ -663,24 +700,32 @@ _svmh_new_native_global_array
 ----------------------------------------------------------------------
 */
 
+#ifdef COMMENT
+
 static jint
 _svmh_new_native_global_array (_svmt_JNIEnv *env, jarray *array)
 {
   return _svmh_new_native_global (env, (jobject *) array);
 }
 
+#endif /* COMMENT */
+
 /*
 ----------------------------------------------------------------------
 _svmh_free_native_global_array
 ----------------------------------------------------------------------
 */
 
+#ifdef COMMENT
+
 static void
 _svmh_free_native_global_array (_svmt_JNIEnv *env, jarray *pglobal_arrayref)
 {
   _svmh_free_native_global (env, (jobject *) pglobal_arrayref);
 }
 
+#endif /* COMMENT */
+
 /*
 ----------------------------------------------------------------------
 _svmh_new_native_local_array
@@ -711,7 +756,7 @@ _svmh_local_wrap_pointer
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmh_local_wrap_pointer (_svmt_JNIEnv *env, void *pointer,
 			  jbyteArray *pwrapper)
 {
@@ -732,7 +777,7 @@ _svmh_local_wrap_pointer (_svmt_JNIEnv *env, void *pointer,
     }
 
   /* put "pointer" into the "wrapper" array */
-  *((void **)
+  *((void **) (void *)
     (((char *) *wrapper) + _svmf_aligned_size_t
      (sizeof (_svmt_array_instance)))) = pointer;
 
@@ -747,7 +792,7 @@ _svmf_wrap_pointer
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_wrap_pointer (_svmt_JNIEnv *env, void *pointer, jbyteArray wrapper)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -760,7 +805,7 @@ _svmf_wrap_pointer (_svmt_JNIEnv *env, void *pointer, jbyteArray wrapper)
     }
 
   /* put "pointer" into the "wrapper" array */
-  *((void **)
+  *((void **) (void *)
     (((char *) *wrapper) + _svmf_aligned_size_t
      (sizeof (_svmt_array_instance)))) = pointer;
 
@@ -776,7 +821,7 @@ _svmf_unwrap_pointer
 static void *
 _svmf_unwrap_pointer (_svmt_array_instance *wrapper)
 {
-  return *((void **)
+  return *((void **) (void *)
 	   (((char *) wrapper) +
 	    _svmf_aligned_size_t (sizeof (_svmt_array_instance))));
 }
@@ -787,7 +832,7 @@ _svmf_unwrap_class_instance
 ----------------------------------------------------------------------
 */
 
-static _svmt_type_info *
+svm_static _svmt_type_info *
 _svmf_unwrap_class_instance (_svmt_JNIEnv *env, jclass class)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -807,8 +852,9 @@ _svmf_new_class
 ----------------------------------------------------------------------
 */
 
-static jint
-_svmf_new_class (_svmt_JNIEnv *env, _svmt_type_info *type)
+svm_static jint
+_svmf_new_class_pd (_svmt_JNIEnv *env, _svmt_type_info *type,
+		    jobject protection_domain)
 {
   _svmt_JavaVM *vm = env->vm;
   jbyteArray data = NULL;
@@ -835,7 +881,8 @@ _svmf_new_class (_svmt_JNIEnv *env, _svmt_type_info *type)
     }
 
   if (_svmm_invoke_nonvirtual_jlclass_init
-      (env, class_instance, _svmf_cast_jobject (data)) != JNI_OK)
+      (env, class_instance, _svmf_cast_jobject (data),
+       protection_domain) != JNI_OK)
     {
       _svmm_free_native_local_array (env, data);
       _svmm_free_native_global (env, class_instance);
@@ -849,6 +896,12 @@ _svmf_new_class (_svmt_JNIEnv *env, _svmt_type_info *type)
   return JNI_OK;
 }
 
+svm_static jint
+_svmf_new_class (_svmt_JNIEnv *env, _svmt_type_info *type)
+{
+  return _svmf_new_class_pd (env, type, (jobject) (NULL));
+}
+
 /*
 ----------------------------------------------------------------------
 _svmf_find_method_in_class
@@ -888,7 +941,7 @@ _svmm_multianewarray
 #define _svmm_multianewarray(env, array_info, dimensions, counts, instance) \
 _svmh_multianewarray (env, array_info, dimensions, &counts, &instance)
 
-static jint
+svm_static jint
 _svmh_multianewarray (_svmt_JNIEnv *env, _svmt_array_info *array_info,
 		      jint dimensions, _svmt_stack_value counts[],
 		      _svmt_array_instance **pinstance)
@@ -935,6 +988,10 @@ _svmh_multianewarray (_svmt_JNIEnv *env, _svmt_array_info *array_info,
 
 	  elements = (_svmt_array_instance **) (*array);
 	  elements[(-1) - i] = instance;
+#if defined (_SABLEVM_GC_WRITE_BARRIER)
+	  _svmf_write_barrier
+	    (env, (_svmt_object_instance **) &elements[(-1) - i]);
+#endif /* _SABLEVM_GC_WRITE_BARRIER */
 
 #else
 
@@ -942,6 +999,9 @@ _svmh_multianewarray (_svmt_JNIEnv *env, _svmt_array_info *array_info,
 	    (((char *) *array) +
 	     _svmf_aligned_size_t (sizeof (_svmt_array_instance)));
 	  elements[i] = instance;
+#if defined (_SABLEVM_GC_WRITE_BARRIER)
+	  _svmf_write_barrier (env, (_svmt_object_instance **) &elements[i]);
+#endif /* _SABLEVM_GC_WRITE_BARRIER */
 
 #endif
 	}
@@ -963,7 +1023,7 @@ _svmf_get_string
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_get_string (_svmt_JNIEnv *env, const char *value, jstring str)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -1005,7 +1065,7 @@ _svmf_get_interned_string
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_get_interned_string (_svmt_JNIEnv *env, const char *value, jstring str)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -1047,7 +1107,7 @@ _svmf_get_current_working_directory
 ----------------------------------------------------------------------
 */
 
-static const char *
+svm_static const char *
 _svmf_get_current_working_directory (void)
 {
   size_t size = 1024;
@@ -1091,7 +1151,7 @@ _svmh_galloc_utf_chars (env, src, &dst)
 #define _svmm_gfree_utf_chars(str) \
 _svmm_gfree_str(str)
 
-static jint
+svm_static jint
 _svmh_galloc_utf_chars (_svmt_JNIEnv *env, jstring src, char **pdst)
 {
   jbyteArray bytes;
@@ -1132,7 +1192,7 @@ _svmf_is_super_class
 ----------------------------------------------------------------------
 */
 
-static jboolean
+svm_static jboolean
 _svmf_is_super_class (_svmt_class_info *class1, _svmt_class_info *class2)
 {
   _svmt_class_info *current;
@@ -1172,7 +1232,7 @@ _svmf_get_current_class_loader
 ----------------------------------------------------------------------
 */
 
-static _svmt_class_loader_info *
+svm_static _svmt_class_loader_info *
 _svmf_get_current_class_loader (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -1188,7 +1248,7 @@ _svmf_get_current_class_loader (_svmt_JNIEnv *env)
 	  return method->class_info->class_loader_info;
 	}
 
-      frame = (_svmt_stack_frame *)
+      frame = (_svmt_stack_frame *) (void *)
 	(((char *) frame) - frame->previous_offset);
       method = frame->method;
     }
@@ -1196,87 +1256,287 @@ _svmf_get_current_class_loader (_svmt_JNIEnv *env)
   return vm->class_loading.boot_loader.class_loader_info;
 }
 
+/*
+----------------------------------------------------------------------
+_svmf_enter_class_loader_monitor
+----------------------------------------------------------------------
+*/
+
+svm_static jint
+_svmf_enter_class_loader_monitor (_svmt_JNIEnv *env,
+				  _svmt_class_loader_info *class_loader_info)
+{
+  if (class_loader_info->class_loader == NULL)
+    {
+      _svmt_JavaVM *vm = env->vm;
+      assert (class_loader_info ==
+	      vm->class_loading.boot_loader.class_loader_info);
+
+      if (_svmf_enter_object_monitor
+	  (env,
+	   *(vm->class_loading.boot_loader.classes.virtualmachine->
+	     class_instance)) != JNI_OK)
+	{
+	  return JNI_ERR;
+	}
+    }
+  else
+    {
+      if (_svmf_enter_object_monitor (env, *(class_loader_info->class_loader))
+	  != JNI_OK)
+	{
+	  return JNI_ERR;
+	}
+    }
+
+  return JNI_OK;
+}
+
+/*
+----------------------------------------------------------------------
+_svmf_exit_class_loader_monitor
+----------------------------------------------------------------------
+*/
+
+svm_static jint
+_svmf_exit_class_loader_monitor (_svmt_JNIEnv *env,
+				 _svmt_class_loader_info *class_loader_info)
+{
+  if (class_loader_info->class_loader == NULL)
+    {
+      _svmt_JavaVM *vm = env->vm;
+      assert (class_loader_info ==
+	      vm->class_loading.boot_loader.class_loader_info);
+
+      if (_svmf_exit_object_monitor
+	  (env,
+	   *(vm->class_loading.boot_loader.classes.virtualmachine->
+	     class_instance)) != JNI_OK)
+	{
+	  return JNI_ERR;
+	}
+    }
+  else
+    {
+      if (_svmf_exit_object_monitor (env, *(class_loader_info->class_loader))
+	  != JNI_OK)
+	{
+	  return JNI_ERR;
+	}
+    }
+
+  return JNI_OK;
+}
+
+/*
+----------------------------------------------------------------------
+_svmf_wrap_exception_in_invocationte
+----------------------------------------------------------------------
+*/
+
+/* This function wraps the "current_exception" parameter into a new
+   InvocationTargetException instance, then throws it. */
+
+svm_static int
+_svmf_wrap_exception_in_invocationte (_svmt_JNIEnv *env,
+				      jobject current_exception)
+{
+  _svmt_JavaVM *vm = env->vm;
+  jobject invocationte = NULL;
+  jint status;
+
+  if (_svmm_new_native_local (env, invocationte) != JNI_OK)
+    {
+      return JNI_ERR;
+    }
+
+  if (_svmm_new_object_instance
+      (env,
+       vm->class_loading.boot_loader.classes.jlrinvocationtargetexception,
+       *invocationte) != JNI_OK)
+    {
+      _svmm_free_native_local (env, invocationte);
+      return JNI_ERR;
+    }
+
+  status =
+    _svmm_invoke_nonvirtual_jlrinvocationtargetexception_init (env,
+							       invocationte,
+							       current_exception);
+
+  /* if no exception was raised, throw InvocationTargetException
+     instance */
+  if (status == JNI_OK)
+    {
+      *(env->throwable) = *invocationte;
+    }
+
+  _svmm_free_native_local (env, invocationte);
+
+  return JNI_ERR;
+}
+
+/*
+----------------------------------------------------------------------
+_svmf_verbose_methods_enter (_svmt_JNIEnv *env)
+----------------------------------------------------------------------
+*/
+
+#ifndef NDEBUG
+
+inline svm_static void
+_svmf_verbose_methods_enter (_svmt_JNIEnv *env)
+{
+  if (env->vm->verbose_methods == JNI_TRUE)
+    {
+      _svmt_method_info *method = env->stack.current_frame->method;
+
+      _svmf_printf (env, stderr,
+		    "T%d: [verbose methods: entering method %s.%s%s]\n",
+		    env->thread.id,
+		    method->class_info->name,
+		    DREF (method->name, value),
+		    DREF (method->descriptor, value));
+    }
+}
+
+#endif /* !NDEBUG */
+
+/*
+----------------------------------------------------------------------
+_svmf_verbose_methods_exit (_svmt_JNIEnv *env)
+----------------------------------------------------------------------
+*/
+
+#ifndef NDEBUG
+
+inline svm_static void
+_svmf_verbose_methods_exit (_svmt_JNIEnv *env)
+{
+  if (env->vm->verbose_methods == JNI_TRUE)
+    {
+      _svmt_method_info *method = env->stack.current_frame->method;
+
+      _svmf_printf (env, stderr,
+		    "T%d: [verbose methods: exiting  method %s.%s%s]\n",
+		    env->thread.id,
+		    method->class_info->name,
+		    DREF (method->name, value),
+		    DREF (method->descriptor, value));
+
+      {
+	_svmt_method_info *caller_method;
+	_svmt_stack_frame *caller_frame = env->stack.current_frame;
+	caller_frame = (_svmt_stack_frame *) (void *)
+	  (((char *) caller_frame) - caller_frame->previous_offset);
+	caller_method = caller_frame->method;
+
+	while (_svmf_is_set_flag
+	       (caller_method->access_flags, SVM_ACC_INTERNAL)
+	       && caller_method != &env->vm->stack_bottom_method)
+	  {
+	    caller_frame = (_svmt_stack_frame *) (void *)
+	      (((char *) caller_frame) - caller_frame->previous_offset);
+	    caller_method = caller_frame->method;
+	  }
+
+	if (!_svmf_is_set_flag
+	    (caller_method->access_flags, SVM_ACC_INTERNAL))
+	  {
+	    _svmf_printf (env, stderr,
+			  "T%d: [                   returning to %s.%s%s]\n",
+			  env->thread.id,
+			  caller_method->class_info->name,
+			  DREF (caller_method->name, value),
+			  DREF (caller_method->descriptor, value));
+	  }
+      }
+    }
+}
+
+#endif /* !NDEBUG */
+
 /*
 ----------------------------------------------------------------------
 _svmf_print_statistics
 ----------------------------------------------------------------------
 */
 #ifdef STATISTICS
-static void
+svm_static void
 _svmf_print_statistics (_svmt_JNIEnv *env)
 {
   _svmt_JavaVM *vm = env->vm;
 
-  _svmf_printf (env, stdout, "\nSTATS: Number of prepared classes: %d\n",
+  _svmf_printf (env, stderr, "\nSTATS: Number of prepared classes: %d\n",
 		vm->class_count);
-  _svmf_printf (env, stdout, "STATS: Number of prepared interfaces: %d\n",
+  _svmf_printf (env, stderr, "STATS: Number of prepared interfaces: %d\n",
 		vm->interface_count);
-  _svmf_printf (env, stdout, "STATS: Number of prepared arrays: %d\n",
+  _svmf_printf (env, stderr, "STATS: Number of prepared arrays: %d\n",
 		vm->array_count);
-  _svmf_printf (env, stdout, "STATS: Number of prepared methods: %d\n",
+  _svmf_printf (env, stderr, "STATS: Number of prepared methods: %d\n",
 		vm->method_count);
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Number of prepared bytecode instructions: %d\n",
 		vm->bytecode_instruction_count);
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Number of prepared bytecodes: %d\n",
 		vm->bytecode_count);
-  _svmf_printf (env, stdout, "STATS: Code array size: %d\n\n", vm->code_size);
+  _svmf_printf (env, stderr, "STATS: Code array size: %d\n\n", vm->code_size);
 
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Maximum stack size: %d\n", vm->max_stack_size);
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Number of locals on stack: %d\n",
 		vm->stack_local_count);
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Number of split locals on stack: %d\n\n",
 		vm->stack_local_split_count);
 
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Total sparse interface virtual tables size: %d\n",
 		vm->total_ivtables);
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Total interface virtual table holes size: %d\n",
 		vm->total_holes);
-  _svmf_printf (env, stdout, "STATS: Total wasted holes size: %d\n\n",
+  _svmf_printf (env, stderr, "STATS: Total wasted holes size: %d\n\n",
 		vm->total_lost_holes);
 
-  _svmf_printf (env, stdout, "STATS: Total GC maps size: %d\n",
+  _svmf_printf (env, stderr, "STATS: Total GC maps size: %d\n",
 		vm->total_gc_maps_size);
-  _svmf_printf (env, stdout, "STATS: GC maps count: %d\n",
+  _svmf_printf (env, stderr, "STATS: GC maps count: %d\n",
 		vm->total_gc_maps_count);
-  _svmf_printf (env, stdout, "STATS: GC points count: %d\n\n",
+  _svmf_printf (env, stderr, "STATS: GC points count: %d\n\n",
 		vm->check_points_count);
 
-  _svmf_printf (env, stdout, "STATS: Original locals count: %d\n",
+  _svmf_printf (env, stderr, "STATS: Original locals count: %d\n",
 		vm->total_orig_locals_count);
-  _svmf_printf (env, stdout, "STATS: Unused locals count: %d\n",
+  _svmf_printf (env, stderr, "STATS: Unused locals count: %d\n",
 		vm->total_unused_locals_count);
-  _svmf_printf (env, stdout, "STATS: Added locals count: %d\n\n",
+  _svmf_printf (env, stderr, "STATS: Added locals count: %d\n\n",
 		vm->total_added_locals_count);
 
 #if defined (_SABLEVM_INLINED_THREADED_INTERPRETER)
 
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Number of distinct (inlined) threads:: %d\n",
 		vm->distinct_inlined_threads_count);
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Total bytecode count of distinct threads: %d\n",
 		vm->total_distinct_thread_bc_length);
-  _svmf_printf (env, stdout,
+  _svmf_printf (env, stderr,
 		"STATS: Total inlined size of distinct threads: %d\n\n",
 		vm->total_distinct_thread_inlined_length);
 
-  _svmf_printf (env, stdout, "STATS: Number of (inlined) threads:: %d\n",
+  _svmf_printf (env, stderr, "STATS: Number of (inlined) threads:: %d\n",
 		vm->inlined_threads_count);
-  _svmf_printf (env, stdout, "STATS: Total bytecode count of threads: %d\n",
+  _svmf_printf (env, stderr, "STATS: Total bytecode count of threads: %d\n",
 		vm->total_thread_bc_length);
-  _svmf_printf (env, stdout, "STATS: Total inlined size of threads: %d\n\n",
+  _svmf_printf (env, stderr, "STATS: Total inlined size of threads: %d\n\n",
 		vm->total_thread_inlined_length);
 
-  _svmf_printf (env, stdout, "STATS: Maximum thread bytecode count: %d\n",
+  _svmf_printf (env, stderr, "STATS: Maximum thread bytecode count: %d\n",
 		vm->max_thread_bc_length);
-  _svmf_printf (env, stdout, "STATS: Maximum thread inlined size: %d\n\n",
+  _svmf_printf (env, stderr, "STATS: Maximum thread inlined size: %d\n\n",
 		vm->max_thread_inlined_length);
 
 #endif
